@@ -3,9 +3,10 @@ from __future__ import division, print_function, absolute_import
 import warnings
 
 import numpy as np
-from scipy.misc import factorial
+from scipy.special import factorial
 
-from scipy.lib.six import xrange
+from scipy._lib.six import xrange
+from scipy._lib._util import _asarray_validated
 
 
 __all__ = ["KroghInterpolator", "krogh_interpolate", "BarycentricInterpolator",
@@ -65,12 +66,12 @@ class _Interpolator1D(object):
 
         Parameters
         ----------
-        x : array-like
+        x : array_like
             Points to evaluate the interpolant at.
 
         Returns
         -------
-        y : array-like
+        y : array_like
             Interpolated values. Shape is determined by replacing
             the interpolation axis in the original array with the shape of x.
 
@@ -87,7 +88,7 @@ class _Interpolator1D(object):
 
     def _prepare_x(self, x):
         """Reshape input x array to 1-D"""
-        x = np.asarray(x)
+        x = _asarray_validated(x, check_finite=False, as_inexact=True)
         x_shape = x.shape
         return x.ravel(), x_shape
 
@@ -148,10 +149,9 @@ class _Interpolator1DWithDerivatives(_Interpolator1D):
 
         Parameters
         ----------
-        x : array-like
+        x : array_like
             Point or points at which to evaluate the derivatives
-
-        der : None or integer
+        der : int or None, optional
             How many derivatives to extract; None for all potentially
             nonzero derivatives (that is a number equal to the number
             of points). This number includes the function value as 0th
@@ -166,6 +166,7 @@ class _Interpolator1DWithDerivatives(_Interpolator1D):
 
         Examples
         --------
+        >>> from scipy.interpolate import KroghInterpolator
         >>> KroghInterpolator([0,0,0],[1,2,3]).derivatives(0)
         array([1.0,2.0,3.0])
         >>> KroghInterpolator([0,0,0],[1,2,3]).derivatives([0,0])
@@ -193,7 +194,7 @@ class _Interpolator1DWithDerivatives(_Interpolator1D):
 
         Parameters
         ----------
-        x : array-like
+        x : array_like
             Point or points at which to evaluate the derivatives
 
         der : integer, optional
@@ -234,9 +235,9 @@ class KroghInterpolator(_Interpolator1DWithDerivatives):
 
     Parameters
     ----------
-    xi : array-like, length N
+    xi : array_like, length N
         Known x-coordinates. Must be sorted in increasing order.
-    yi : array-like
+    yi : array_like
         Known y-coordinates. When an xi occurs two or more times in
         a row, the corresponding yi's represent derivative values.
     axis : int, optional
@@ -265,6 +266,7 @@ class KroghInterpolator(_Interpolator1DWithDerivatives):
     To produce a polynomial that is zero at 0 and 1 and has
     derivative 2 at 0, call
 
+    >>> from scipy.interpolate import KroghInterpolator
     >>> KroghInterpolator([0,0,1],[0,2,0])
 
     This constructs the quadratic 2*X**2-2*X. The derivative condition
@@ -274,6 +276,8 @@ class KroghInterpolator(_Interpolator1DWithDerivatives):
     For another example, given xi, yi, and a derivative ypi for each
     point, appropriate arrays can be constructed as:
 
+    >>> xi = np.linspace(0, 1, 5)
+    >>> yi, ypi = np.random.rand(2, 5)
     >>> xi_k, yi_k = np.repeat(xi, 2), np.ravel(np.dstack((yi,ypi)))
     >>> KroghInterpolator(xi_k, yi_k)
 
@@ -352,7 +356,7 @@ class KroghInterpolator(_Interpolator1DWithDerivatives):
         return cn[:der]
 
 
-def krogh_interpolate(xi,yi,x,der=0,axis=0):
+def krogh_interpolate(xi, yi, x, der=0, axis=0):
     """
     Convenience function for polynomial interpolation.
 
@@ -367,7 +371,7 @@ def krogh_interpolate(xi,yi,x,der=0,axis=0):
         vectors of length R, or scalars if R=1.
     x : array_like
         Point or points at which to evaluate the derivatives.
-    der : int or list
+    der : int or list, optional
         How many derivatives to extract; None for all potentially
         nonzero derivatives (that is a number equal to the number
         of points), or a list of derivatives to extract. This number
@@ -473,10 +477,10 @@ class BarycentricInterpolator(_Interpolator1D):
 
     Parameters
     ----------
-    xi : array-like
+    xi : array_like
         1-d array of x coordinates of the points the polynomial
         should pass through
-    yi : array-like
+    yi : array_like, optional
         The y coordinates of the points the polynomial should pass through.
         If None, the y values will be supplied later via the `set_y` method.
     axis : int, optional
@@ -578,12 +582,12 @@ class BarycentricInterpolator(_Interpolator1D):
 
         Parameters
         ----------
-        x : array-like
+        x : array_like
             Points to evaluate the interpolant at.
 
         Returns
         -------
-        y : array-like
+        y : array_like
             Interpolated values. Shape is determined by replacing
             the interpolation axis in the original array with the shape of x.
 
@@ -676,32 +680,35 @@ class PiecewisePolynomial(_Interpolator1DWithDerivatives):
 
     Parameters
     ----------
-    xi : array-like
-        a sorted 1-d array of x-coordinates
-    yi : array-like or list of array-likes
-        yi[i][j] is the j-th derivative known at xi[i]   (for axis=0)
-    orders : list of integers, or integer
-        a list of polynomial orders, or a single universal order
-    direction : {None, 1, -1}
-        indicates whether the xi are increasing or decreasing
-        +1 indicates increasing
-        -1 indicates decreasing
-        None indicates that it should be deduced from the first two xi
+    xi : array_like
+        A sorted 1-d array of x-coordinates.
+    yi : array_like or list of array_likes
+        ``yi[i][j]`` is the j-th derivative known at ``xi[i]`` (for ``axis=0``).
+    orders : list of int, or int, optional
+        A list of polynomial orders, or a single universal order.
+    direction : {None, 1, -1}, optional
+        Indicates whether the `xi` are increasing or decreasing:
+
+          +1 : increasing values
+
+          -1 : decreasing values
+
+          None : direction will be deduced from the first two elements of `xi`
+
     axis : int, optional
-        Axis in the yi array corresponding to the x-coordinate values.
+        Axis in the `yi` array corresponding to the x-coordinate values.
 
     Notes
     -----
-    If orders is None, or orders[i] is None, then the degree of the
-    polynomial segment is exactly the degree required to match all i
-    available derivatives at both endpoints. If orders[i] is not None,
+    If orders is None, or ``orders[i]`` is None, then the degree of the
+    polynomial segment is exactly the degree required to match all ``i``
+    available derivatives at both endpoints.  If ``orders[i]`` is not None,
     then some derivatives will be ignored. The code will try to use an
     equal number of derivatives from each end; if the total number of
     derivatives needed is odd, it will prefer the rightmost endpoint. If
     not enough derivatives are available, an exception is raised.
 
     """
-
     def __init__(self, xi, yi, orders=None, direction=None, axis=0):
         _Interpolator1DWithDerivatives.__init__(self, axis=axis)
 
@@ -751,7 +758,10 @@ class PiecewisePolynomial(_Interpolator1DWithDerivatives):
         n2 = min(n-n1,len(y2))
         n1 = min(n-n2,len(y1))
         if n1+n2 != n:
-            raise ValueError("Point %g has %d derivatives, point %g has %d derivatives, but order %d requested" % (x1, len(y1), x2, len(y2), order))
+            raise ValueError("Point %g has %d derivatives, point %g has %d "
+                             "derivatives, but order %d requested" % (x1,
+                             len(y1), x2, len(y2), order))
+
         if not (n1 <= len(y1) and n2 <= len(y2)):
             raise ValueError("`order` input incompatible with length y1 or y2.")
 
@@ -772,12 +782,12 @@ class PiecewisePolynomial(_Interpolator1DWithDerivatives):
         Parameters
         ----------
         xi : float
-            Input
+            Point to add.
         yi : array_like
-            `yi` is the list of derivatives known at `xi`
-        order : integer or None
-            a polynomial order, or instructions to use the highest
-            possible order
+            `yi` is the list of derivatives known at `xi`.
+        order : int or None, optional
+            A polynomial order, or instructions to use the highest possible
+            order.
 
         """
         yi = self._reshape_yi(yi, check=True)
@@ -786,7 +796,8 @@ class PiecewisePolynomial(_Interpolator1DWithDerivatives):
         if self.direction is None:
             self.direction = np.sign(xi-self.xi[-1])
         elif (xi-self.xi[-1])*self.direction < 0:
-            raise ValueError("x coordinates must be in the %d direction: %s" % (self.direction, self.xi))
+            raise ValueError("x coordinates must be in the %d direction: %s" % (
+                             self.direction, self.xi))
 
         self.xi.append(xi)
         self.yi.append(yi)
@@ -815,16 +826,8 @@ class PiecewisePolynomial(_Interpolator1DWithDerivatives):
         yi : list of lists of length N1
             ``yi[i]`` (if ``axis == 0``) is the list of derivatives known
             at ``xi[i]``.
-        orders : int or list of ints
+        orders : int or list of ints, optional
             A list of polynomial orders, or a single universal order.
-        direction : {None, 1, -1}
-            Indicates whether the `xi` are increasing or decreasing.
-
-                +1 indicates increasing
-
-                -1 indicates decreasing
-
-            None indicates that it should be deduced from the first two `xi`.
 
         """
         if self._y_axis == 0:
@@ -873,7 +876,7 @@ class PiecewisePolynomial(_Interpolator1DWithDerivatives):
         return y
 
 
-def piecewise_polynomial_interpolate(xi,yi,x,orders=None,der=0,axis=0):
+def piecewise_polynomial_interpolate(xi, yi, x, orders=None, der=0, axis=0):
     """
     Convenience function for piecewise polynomial interpolation.
 
@@ -887,7 +890,7 @@ def piecewise_polynomial_interpolate(xi,yi,x,orders=None,der=0,axis=0):
         Coordinates at which to evalualte the polynomial.
     orders : int or list of ints, optional
         A list of polynomial orders, or a single universal order.
-    der : int or list
+    der : int or list, optional
         How many derivatives to extract; None for all potentially
         nonzero derivatives (that is a number equal to the number
         of points), or a list of derivatives to extract. This number
