@@ -14,17 +14,17 @@
 
 from __future__ import division, print_function, absolute_import
 
-__all__ = ['eig','eigh','eig_banded','eigvals','eigvalsh', 'eigvals_banded',
-           'hessenberg']
+__all__ = ['eig', 'eigh', 'eig_banded', 'eigvals', 'eigvalsh',
+           'eigvals_banded', 'hessenberg']
 
 import numpy
-from numpy import array, asarray_chkfinite, asarray, diag, zeros, ones, \
-        isfinite, inexact, nonzero, iscomplexobj, cast, flatnonzero, conj
+from numpy import (array, isfinite, inexact, nonzero, iscomplexobj, cast,
+                   flatnonzero, conj)
 # Local imports
-from scipy.linalg import calc_lwork
-from .misc import LinAlgError, _datacopied
-from .lapack import get_lapack_funcs
-from .blas import get_blas_funcs
+from scipy._lib.six import xrange
+from scipy._lib._util import _asarray_validated
+from .misc import LinAlgError, _datacopied, norm
+from .lapack import get_lapack_funcs, _compute_lwork
 
 
 _I = cast['F'](1j)
@@ -39,8 +39,8 @@ def _make_complex_eigvecs(w, vin, dtype):
     m = (w.imag > 0)
     m[:-1] |= (w.imag[1:] < 0)  # workaround for LAPACK bug, cf. ticket #709
     for i in flatnonzero(m):
-        v.imag[:,i] = vin[:,i+1]
-        conj(v[:,i], v[:,i+1])
+        v.imag[:, i] = vin[:, i+1]
+        conj(v[:, i], v[:, i+1])
     return v
 
 
@@ -51,18 +51,19 @@ def _geneig(a1, b1, left, right, overwrite_a, overwrite_b):
     lwork = res[-2][0].real.astype(numpy.int)
     if ggev.typecode in 'cz':
         alpha, beta, vl, vr, work, info = ggev(a1, b1, cvl, cvr, lwork,
-                                                    overwrite_a, overwrite_b)
+                                               overwrite_a, overwrite_b)
         w = alpha / beta
     else:
-        alphar, alphai, beta, vl, vr, work, info = ggev(a1, b1, cvl, cvr, lwork,
-                                                        overwrite_a,overwrite_b)
+        alphar, alphai, beta, vl, vr, work, info = ggev(a1, b1, cvl, cvr,
+                                                        lwork, overwrite_a,
+                                                        overwrite_b)
         w = (alphar + _I * alphai) / beta
     if info < 0:
-        raise ValueError('illegal value in %d-th argument of internal ggev'
-                                                                    % -info)
+        raise ValueError('illegal value in %d-th argument of internal ggev' %
+                         -info)
     if info > 0:
-        raise LinAlgError("generalized eig algorithm did not converge (info=%d)"
-                                                                    % info)
+        raise LinAlgError("generalized eig algorithm did not converge "
+                          "(info=%d)" % info)
 
     only_real = numpy.logical_and.reduce(numpy.equal(w.imag, 0.0))
     if not (ggev.typecode in 'cz' or only_real):
@@ -71,6 +72,14 @@ def _geneig(a1, b1, left, right, overwrite_a, overwrite_b):
             vl = _make_complex_eigvecs(w, vl, t)
         if right:
             vr = _make_complex_eigvecs(w, vr, t)
+
+    # the eigenvectors returned by the lapack function are NOT normalized
+    for i in xrange(vr.shape[0]):
+        if right:
+            vr[:, i] /= norm(vr[:, i])
+        if left:
+            vl[:, i] /= norm(vl[:, i])
+
     if not (left or right):
         return w
     if left:
@@ -108,7 +117,7 @@ def eig(a, b=None, left=False, right=True, overwrite_a=False,
         Whether to overwrite `a`; may improve performance.  Default is False.
     overwrite_b : bool, optional
         Whether to overwrite `b`; may improve performance.  Default is False.
-    check_finite : boolean, optional
+    check_finite : bool, optional
         Whether to check that the input matrices contain only finite numbers.
         Disabling may give a performance gain, but may result in problems
         (crashes, non-termination) if the inputs do contain infinities or NaNs.
@@ -119,7 +128,7 @@ def eig(a, b=None, left=False, right=True, overwrite_a=False,
         The eigenvalues, each repeated according to its multiplicity.
     vl : (M, M) double or complex ndarray
         The normalized left eigenvector corresponding to the eigenvalue
-        ``w[i]`` is the column v[:,i]. Only returned if ``left=True``.
+        ``w[i]`` is the column vl[:,i]. Only returned if ``left=True``.
     vr : (M, M) double or complex ndarray
         The normalized right eigenvector corresponding to the eigenvalue
         ``w[i]`` is the column ``vr[:,i]``.  Only returned if ``right=True``.
@@ -134,48 +143,45 @@ def eig(a, b=None, left=False, right=True, overwrite_a=False,
     eigh : Eigenvalues and right eigenvectors for symmetric/Hermitian arrays.
 
     """
-    if check_finite:
-        a1 = asarray_chkfinite(a)
-    else:
-        a1 = asarray(a)
+    a1 = _asarray_validated(a, check_finite=check_finite)
     if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
         raise ValueError('expected square matrix')
     overwrite_a = overwrite_a or (_datacopied(a1, a))
     if b is not None:
-        if check_finite:
-            b1 = asarray_chkfinite(b)
-        else:
-            b1 = asarray(b)
+        b1 = _asarray_validated(b, check_finite=check_finite)
         overwrite_b = overwrite_b or _datacopied(b1, b)
         if len(b1.shape) != 2 or b1.shape[0] != b1.shape[1]:
             raise ValueError('expected square matrix')
         if b1.shape != a1.shape:
             raise ValueError('a and b must have the same shape')
         return _geneig(a1, b1, left, right, overwrite_a, overwrite_b)
-    geev, = get_lapack_funcs(('geev',), (a1,))
+
+    geev, geev_lwork = get_lapack_funcs(('geev', 'geev_lwork'), (a1,))
     compute_vl, compute_vr = left, right
 
-    lwork = calc_lwork.geev(geev.typecode, a1.shape[0],
-                                compute_vl, compute_vr)[1]
+    lwork = _compute_lwork(geev_lwork, a1.shape[0],
+                           compute_vl=compute_vl,
+                           compute_vr=compute_vr)
+
     if geev.typecode in 'cz':
         w, vl, vr, info = geev(a1, lwork=lwork,
-                                    compute_vl=compute_vl,
-                                    compute_vr=compute_vr,
-                                    overwrite_a=overwrite_a)
+                               compute_vl=compute_vl,
+                               compute_vr=compute_vr,
+                               overwrite_a=overwrite_a)
     else:
         wr, wi, vl, vr, info = geev(a1, lwork=lwork,
                                     compute_vl=compute_vl,
                                     compute_vr=compute_vr,
                                     overwrite_a=overwrite_a)
-        t = {'f':'F','d':'D'}[wr.dtype.char]
+        t = {'f': 'F', 'd': 'D'}[wr.dtype.char]
         w = wr + _I * wi
 
     if info < 0:
-        raise ValueError('illegal value in %d-th argument of internal geev'
-                                                                    % -info)
+        raise ValueError('illegal value in %d-th argument of internal geev' %
+                         -info)
     if info > 0:
         raise LinAlgError("eig algorithm did not converge (only eigenvalues "
-                            "with order >= %d have converged)" % info)
+                          "with order >= %d have converged)" % info)
 
     only_real = numpy.logical_and.reduce(numpy.equal(w.imag, 0.0))
     if not (geev.typecode in 'cz' or only_real):
@@ -240,7 +246,7 @@ def eigh(a, b=None, lower=True, eigvals_only=False, overwrite_a=False,
         Whether to overwrite data in `a` (may improve performance)
     overwrite_b : bool, optional
         Whether to overwrite data in `b` (may improve performance)
-    check_finite : boolean, optional
+    check_finite : bool, optional
         Whether to check that the input matrices contain only finite numbers.
         Disabling may give a performance gain, but may result in problems
         (crashes, non-termination) if the inputs do contain infinities or NaNs.
@@ -279,10 +285,7 @@ def eigh(a, b=None, lower=True, eigvals_only=False, overwrite_a=False,
     eig : eigenvalues and right eigenvectors for non-symmetric arrays
 
     """
-    if check_finite:
-        a1 = asarray_chkfinite(a)
-    else:
-        a1 = asarray(a)
+    a1 = _asarray_validated(a, check_finite=check_finite)
     if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
         raise ValueError('expected square matrix')
     overwrite_a = overwrite_a or (_datacopied(a1, a))
@@ -291,10 +294,7 @@ def eigh(a, b=None, lower=True, eigvals_only=False, overwrite_a=False,
     else:
         cplx = False
     if b is not None:
-        if check_finite:
-            b1 = asarray_chkfinite(b)
-        else:
-            b1 = asarray(b)
+        b1 = _asarray_validated(b, check_finite=check_finite)
         overwrite_b = overwrite_b or _datacopied(b1, b)
         if len(b1.shape) != 2 or b1.shape[0] != b1.shape[1]:
             raise ValueError('expected square matrix')
@@ -353,22 +353,22 @@ def eigh(a, b=None, lower=True, eigvals_only=False, overwrite_a=False,
     else:
         # Use '*gvx' routines if range is specified
         if eigvals is not None:
-            (gvx,) = get_lapack_funcs((pfx+'gvx',), (a1,b1))
+            (gvx,) = get_lapack_funcs((pfx+'gvx',), (a1, b1))
             (lo, hi) = eigvals
             w_tot, v, ifail, info = gvx(a1, b1, uplo=uplo, iu=hi,
-                                        itype=type,jobz=_job, il=lo,
+                                        itype=type, jobz=_job, il=lo,
                                         overwrite_a=overwrite_a,
                                         overwrite_b=overwrite_b)
             w = w_tot[0:hi-lo+1]
         # Use '*gvd' routine if turbo is on and no eigvals are specified
         elif turbo:
-            (gvd,) = get_lapack_funcs((pfx+'gvd',), (a1,b1))
+            (gvd,) = get_lapack_funcs((pfx+'gvd',), (a1, b1))
             v, w, info = gvd(a1, b1, uplo=uplo, itype=type, jobz=_job,
                              overwrite_a=overwrite_a,
                              overwrite_b=overwrite_b)
         # Use '*gv' routine if turbo is off and no eigvals are specified
         else:
-            (gv,) = get_lapack_funcs((pfx+'gv',), (a1,b1))
+            (gv,) = get_lapack_funcs((pfx+'gv',), (a1, b1))
             v, w, info = gv(a1, b1, uplo=uplo, itype=type, jobz=_job,
                             overwrite_a=overwrite_a,
                             overwrite_b=overwrite_b)
@@ -387,7 +387,7 @@ def eigh(a, b=None, lower=True, eigvals_only=False, overwrite_a=False,
         raise LinAlgError("unrecoverable internal error.")
 
     # The algorithm failed to converge.
-    elif info > 0 and info <= b1.shape[0]:
+    elif 0 < info <= b1.shape[0]:
         if eigvals is not None:
             raise LinAlgError("the eigenvectors %s failed to"
                               " converge." % nonzero(ifail)-1)
@@ -467,7 +467,7 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
 
         In doubt, leave this parameter untouched.
 
-    check_finite : boolean, optional
+    check_finite : bool, optional
         Whether to check that the input matrix contains only finite numbers.
         Disabling may give a performance gain, but may result in problems
         (crashes, non-termination) if the inputs do contain infinities or NaNs.
@@ -485,10 +485,7 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
 
     """
     if eigvals_only or overwrite_a_band:
-        if check_finite:
-            a1 = asarray_chkfinite(a_band)
-        else:
-            a1 = asarray(a_band)
+        a1 = _asarray_validated(a_band, check_finite=check_finite)
         overwrite_a_band = overwrite_a_band or (_datacopied(a1, a_band))
     else:
         a1 = array(a_band)
@@ -514,9 +511,8 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
             #         see above
             # lwork = calc_lwork.sbevd(bevd.typecode, a1.shape[0], lower)
             internal_name = 'sbevd'
-        w,v,info = bevd(a1, compute_v=not eigvals_only,
-                        lower=lower,
-                        overwrite_ab=overwrite_a_band)
+        w, v, info = bevd(a1, compute_v=not eigvals_only,
+                          lower=lower, overwrite_ab=overwrite_a_band)
     if select.lower() in [1, 2, 'i', 'v', 'index', 'value']:
         # calculate certain range only
         if select.lower() in [2, 'i', 'index']:
@@ -557,8 +553,8 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
         if not eigvals_only:
             v = v[:, :m]
     if info < 0:
-        raise ValueError('illegal value in %d-th argument of internal %s'
-                                                    % (-info, internal_name))
+        raise ValueError('illegal value in %d-th argument of internal %s' %
+                         (-info, internal_name))
     if info > 0:
         raise LinAlgError("eig algorithm did not converge")
 
@@ -583,9 +579,9 @@ def eigvals(a, b=None, overwrite_a=False, check_finite=True):
     b : (M, M) array_like, optional
         Right-hand side matrix in a generalized eigenvalue problem.
         If omitted, identity matrix is assumed.
-    overwrite_a : boolean, optional
+    overwrite_a : bool, optional
         Whether to overwrite data in a (may improve performance)
-    check_finite : boolean, optional
+    check_finite : bool, optional
         Whether to check that the input matrices contain only finite numbers.
         Disabling may give a performance gain, but may result in problems
         (crashes, non-termination) if the inputs do contain infinities or NaNs.
@@ -609,7 +605,7 @@ def eigvals(a, b=None, overwrite_a=False, check_finite=True):
 
     """
     return eig(a, b=b, left=0, right=0, overwrite_a=overwrite_a,
-                check_finite=check_finite)
+               check_finite=check_finite)
 
 
 def eigvalsh(a, b=None, lower=True, overwrite_a=False,
@@ -644,7 +640,7 @@ def eigvalsh(a, b=None, lower=True, overwrite_a=False,
         Indexes of the smallest and largest (in ascending order) eigenvalues
         and corresponding eigenvectors to be returned: 0 <= lo < hi <= M-1.
         If omitted, all eigenvalues and eigenvectors are returned.
-    type : integer, optional
+    type : int, optional
         Specifies the problem type to be solved:
 
            type = 1: a   v[:,i] = w[i] b v[:,i]
@@ -656,7 +652,7 @@ def eigvalsh(a, b=None, lower=True, overwrite_a=False,
         Whether to overwrite data in `a` (may improve performance)
     overwrite_b : bool, optional
         Whether to overwrite data in `b` (may improve performance)
-    check_finite : boolean, optional
+    check_finite : bool, optional
         Whether to check that the input matrices contain only finite numbers.
         Disabling may give a performance gain, but may result in problems
         (crashes, non-termination) if the inputs do contain infinities or NaNs.
@@ -724,11 +720,11 @@ def eigvals_banded(a_band, lower=False, overwrite_a_band=False,
     ----------
     a_band : (u+1, M) array_like
         The bands of the M by M matrix a.
-    lower : boolean
+    lower : bool, optional
         Is the matrix in the lower form. (Default is upper form)
-    overwrite_a_band:
+    overwrite_a_band : bool, optional
         Discard data in a_band (may enhance performance)
-    select : {'a', 'v', 'i'}
+    select : {'a', 'v', 'i'}, optional
         Which eigenvalues to calculate
 
         ======  ========================================
@@ -738,9 +734,9 @@ def eigvals_banded(a_band, lower=False, overwrite_a_band=False,
         'v'     Eigenvalues in the interval (min, max]
         'i'     Eigenvalues with indices min <= i <= max
         ======  ========================================
-    select_range : (min, max)
+    select_range : (min, max), optional
         Range of selected eigenvalues
-    check_finite : boolean, optional
+    check_finite : bool, optional
         Whether to check that the input matrix contains only finite numbers.
         Disabling may give a performance gain, but may result in problems
         (crashes, non-termination) if the inputs do contain infinities or NaNs.
@@ -766,7 +762,7 @@ def eigvals_banded(a_band, lower=False, overwrite_a_band=False,
                       overwrite_a_band=overwrite_a_band, select=select,
                       select_range=select_range, check_finite=check_finite)
 
-_double_precision = ['i','l','d']
+_double_precision = ['i', 'l', 'd']
 
 
 def hessenberg(a, calc_q=False, overwrite_a=False, check_finite=True):
@@ -789,7 +785,7 @@ def hessenberg(a, calc_q=False, overwrite_a=False, check_finite=True):
     overwrite_a : bool, optional
         Whether to overwrite `a`; may improve performance.
         Default is False.
-    check_finite : boolean, optional
+    check_finite : bool, optional
         Whether to check that the input matrix contains only finite numbers.
         Disabling may give a performance gain, but may result in problems
         (crashes, non-termination) if the inputs do contain infinities or NaNs.
@@ -803,46 +799,41 @@ def hessenberg(a, calc_q=False, overwrite_a=False, check_finite=True):
         Only returned if ``calc_q=True``.
 
     """
-    if check_finite:
-        a1 = asarray_chkfinite(a)
-    else:
-        a1 = asarray(a)
+    a1 = _asarray_validated(a, check_finite=check_finite)
     if len(a1.shape) != 2 or (a1.shape[0] != a1.shape[1]):
         raise ValueError('expected square matrix')
     overwrite_a = overwrite_a or (_datacopied(a1, a))
-    gehrd,gebal = get_lapack_funcs(('gehrd','gebal'), (a1,))
-    ba, lo, hi, pivscale, info = gebal(a1, permute=1, overwrite_a=overwrite_a)
+
+    # if 2x2 or smaller: already in Hessenberg
+    if a1.shape[0] <= 2:
+        if calc_q:
+            return a1, numpy.eye(a1.shape[0])
+        return a1
+
+    gehrd, gebal, gehrd_lwork = get_lapack_funcs(('gehrd', 'gebal',
+                                                  'gehrd_lwork'), (a1,))
+    ba, lo, hi, pivscale, info = gebal(a1, permute=0, overwrite_a=overwrite_a)
     if info < 0:
         raise ValueError('illegal value in %d-th argument of internal gebal '
-                                                    '(hessenberg)' % -info)
+                         '(hessenberg)' % -info)
     n = len(a1)
-    lwork = calc_lwork.gehrd(gehrd.typecode, n, lo, hi)
+
+    lwork = _compute_lwork(gehrd_lwork, ba.shape[0], lo=lo, hi=hi)
+
     hq, tau, info = gehrd(ba, lo=lo, hi=hi, lwork=lwork, overwrite_a=1)
     if info < 0:
         raise ValueError('illegal value in %d-th argument of internal gehrd '
-                                        '(hessenberg)' % -info)
-
+                         '(hessenberg)' % -info)
+    h = numpy.triu(hq, -1)
     if not calc_q:
-        for i in range(lo, hi):
-            hq[i+2:hi+1, i] = 0.0
-        return hq
+        return h
 
-    # XXX: Use ORGHR routines to compute q.
-    typecode = hq.dtype
-    ger,gemm = get_blas_funcs(('ger','gemm'), dtype=typecode)
-    q = None
-    for i in range(lo, hi):
-        if tau[i] == 0.0:
-            continue
-        v = zeros(n, dtype=typecode)
-        v[i+1] = 1.0
-        v[i+2:hi+1] = hq[i+2:hi+1, i]
-        hq[i+2:hi+1, i] = 0.0
-        h = ger(-tau[i], v, v,a=diag(ones(n, dtype=typecode)), overwrite_a=1)
-        if q is None:
-            q = h
-        else:
-            q = gemm(1.0, q, h)
-    if q is None:
-        q = diag(ones(n, dtype=typecode))
-    return hq, q
+    # use orghr/unghr to compute q
+    orghr, orghr_lwork = get_lapack_funcs(('orghr', 'orghr_lwork'), (a1,))
+    lwork = _compute_lwork(orghr_lwork, n, lo=lo, hi=hi)
+
+    q, info = orghr(a=hq, tau=tau, lo=lo, hi=hi, lwork=lwork, overwrite_a=1)
+    if info < 0:
+        raise ValueError('illegal value in %d-th argument of internal orghr '
+                         '(hessenberg)' % -info)
+    return h, q
