@@ -14,13 +14,15 @@ systems that don't have PIL installed.
 .. autosummary::
    :toctree: generated/
 
+   ascent - Get example image for processing
    bytescale - Byte scales an array (image)
    central_diff_weights - Weights for an n-point central m-th derivative
-   comb - Combinations of N things taken k at a time, "N choose k"
+   comb - Combinations of N things taken k at a time, "N choose k" (imported from scipy.special)
    derivative - Find the n-th derivative of a function at a point
-   factorial  - The factorial function, n! = special.gamma(n+1)
-   factorial2 - Double factorial, (n!)!
-   factorialk - (...((n!)!)!...)! where there are k '!'
+   face - Get example image for processing
+   factorial  - The factorial function, n! = special.gamma(n+1) (imported from scipy.special)
+   factorial2 - Double factorial, (n!)! (imported from scipy.special)
+   factorialk - (...((n!)!)!...)! where there are k '!' (imported from scipy.special)
    fromimage - Return a copy of a PIL image as a numpy array
    imfilter - Simple filtering of an image
    imread - Read an image file from a filename
@@ -33,17 +35,20 @@ systems that don't have PIL installed.
    logsumexp - Compute the log of the sum of exponentials of input elements
    pade - Pade approximation to function as the ratio of two polynomials
    toimage - Takes a numpy array and returns a PIL image
+   source - Print function source code
    who - Print the Numpy arrays in the given dictionary
 
 """
 
 from __future__ import division, print_function, absolute_import
 
-__all__ = ['who', 'source', 'info', 'doccer']
+__all__ = ['who', 'source', 'info', 'doccer',
+           'comb', 'factorial', 'factorial2', 'factorialk']
 
 from . import doccer
 from .common import *
 from numpy import who, source, info as _info
+from scipy.special import comb, factorial, factorial2, factorialk
 
 import sys
 
