@@ -3,14 +3,19 @@ Unit tests for optimization routines from minpack.py.
 """
 from __future__ import division, print_function, absolute_import
 
-from numpy.testing import assert_, assert_almost_equal, assert_array_equal, \
-        assert_array_almost_equal, TestCase, run_module_suite, assert_raises, \
-        assert_allclose
+import warnings
+
+from numpy.testing import (assert_, assert_almost_equal, assert_array_equal,
+        assert_array_almost_equal, TestCase, run_module_suite, assert_raises,
+        assert_allclose)
 import numpy as np
 from numpy import array, float64, matrix
 
 from scipy import optimize
+from scipy.special import lambertw
 from scipy.optimize.minpack import leastsq, curve_fit, fixed_point
+from scipy._lib._numpy_compat import _assert_warns
+from scipy.optimize import OptimizeWarning
 
 
 class ReturnShape(object):
@@ -91,13 +96,13 @@ def pressure_network_jacobian(flow_rates, Qtot, k):
 
 
 def pressure_network_fun_and_grad(flow_rates, Qtot, k):
-    return pressure_network(flow_rates, Qtot, k), \
-        pressure_network_jacobian(flow_rates, Qtot, k)
+    return (pressure_network(flow_rates, Qtot, k),
+            pressure_network_jacobian(flow_rates, Qtot, k))
 
 
 class TestFSolve(TestCase):
     def test_pressure_network_no_gradient(self):
-        """fsolve without gradient, equal pipes -> equal flows"""
+        # fsolve without gradient, equal pipes -> equal flows.
         k = np.ones(4) * 0.5
         Qtot = 4
         initial_guess = array([2., 0., 2., 0.])
@@ -108,7 +113,7 @@ class TestFSolve(TestCase):
         assert_(ier == 1, mesg)
 
     def test_pressure_network_with_gradient(self):
-        """fsolve with gradient, equal pipes -> equal flows"""
+        # fsolve with gradient, equal pipes -> equal flows
         k = np.ones(4) * 0.5
         Qtot = 4
         initial_guess = array([2., 0., 2., 0.])
@@ -118,7 +123,6 @@ class TestFSolve(TestCase):
         assert_array_almost_equal(final_flows, np.ones(4))
 
     def test_wrong_shape_func_callable(self):
-        """The callable 'func' has no '__name__' attribute."""
         func = ReturnShape(1)
         # x0 is a list of two elements, but func will return an array with
         # length 1, so this should result in a TypeError.
@@ -132,7 +136,6 @@ class TestFSolve(TestCase):
         assert_raises(TypeError, optimize.fsolve, dummy_func, x0, args=((1,),))
 
     def test_wrong_shape_fprime_callable(self):
-        """The callables 'func' and 'deriv_func' have no '__name__' attribute."""
         func = ReturnShape(1)
         deriv_func = ReturnShape((2,2))
         assert_raises(TypeError, optimize.fsolve, func, x0=[0,1], fprime=deriv_func)
@@ -150,7 +153,7 @@ class TestFSolve(TestCase):
 
 class TestRootHybr(TestCase):
     def test_pressure_network_no_gradient(self):
-        """root/hybr without gradient, equal pipes -> equal flows"""
+        # root/hybr without gradient, equal pipes -> equal flows
         k = np.ones(4) * 0.5
         Qtot = 4
         initial_guess = array([2., 0., 2., 0.])
@@ -159,7 +162,7 @@ class TestRootHybr(TestCase):
         assert_array_almost_equal(final_flows, np.ones(4))
 
     def test_pressure_network_with_gradient(self):
-        """root/hybr with gradient, equal pipes -> equal flows"""
+        # root/hybr with gradient, equal pipes -> equal flows
         k = np.ones(4) * 0.5
         Qtot = 4
         initial_guess = matrix([2., 0., 2., 0.])
@@ -169,7 +172,8 @@ class TestRootHybr(TestCase):
         assert_array_almost_equal(final_flows, np.ones(4))
 
     def test_pressure_network_with_gradient_combined(self):
-        """root/hybr with gradient and function combined, equal pipes -> equal flows"""
+        # root/hybr with gradient and function combined, equal pipes -> equal
+        # flows
         k = np.ones(4) * 0.5
         Qtot = 4
         initial_guess = array([2., 0., 2., 0.])
@@ -181,7 +185,7 @@ class TestRootHybr(TestCase):
 
 class TestRootLM(TestCase):
     def test_pressure_network_no_gradient(self):
-        """root/lm without gradient, equal pipes -> equal flows"""
+        # root/lm without gradient, equal pipes -> equal flows
         k = np.ones(4) * 0.5
         Qtot = 4
         initial_guess = array([2., 0., 2., 0.])
@@ -232,7 +236,6 @@ class TestLeastSq(TestCase):
         assert_array_equal(p0, p0_copy)
 
     def test_wrong_shape_func_callable(self):
-        """The callable 'func' has no '__name__' attribute."""
         func = ReturnShape(1)
         # x0 is a list of two elements, but func will return an array with
         # length 1, so this should result in a TypeError.
@@ -246,7 +249,6 @@ class TestLeastSq(TestCase):
         assert_raises(TypeError, optimize.leastsq, dummy_func, x0, args=((1,),))
 
     def test_wrong_shape_Dfun_callable(self):
-        """The callables 'func' and 'deriv_func' have no '__name__' attribute."""
         func = ReturnShape(1)
         deriv_func = ReturnShape((2,2))
         assert_raises(TypeError, optimize.leastsq, func, x0=[0,1], Dfun=deriv_func)
@@ -257,7 +259,7 @@ class TestLeastSq(TestCase):
         assert_raises(TypeError, optimize.leastsq, func, x0=[0,1], Dfun=deriv_func)
 
     def test_float32(self):
-        # From Track ticket #920
+        # Regression test for gh-1447
         def func(p,x,y):
             q = p[0]*np.exp(-(x-p[1])**2/(2.0*p[2]**2))+p[3]
             return q - y
@@ -299,7 +301,7 @@ class TestCurveFit(TestCase):
         assert_(len(popt) == 2)
         assert_(pcov.shape == (2,2))
         assert_array_almost_equal(popt, [1.7989, 1.1642], decimal=4)
-        assert_array_almost_equal(pcov, [[0.0852, -0.1260],[-0.1260, 0.1912]],
+        assert_array_almost_equal(pcov, [[0.0852, -0.1260], [-0.1260, 0.1912]],
                                   decimal=4)
 
     def test_func_is_classmethod(self):
@@ -344,37 +346,44 @@ class TestCurveFit(TestCase):
         def f(x, a, b):
             return a*x + b
 
-        popt, pcov = curve_fit(f, xdata, ydata, p0=[2, 0], sigma=sigma)
-        perr_scaled = np.sqrt(np.diag(pcov))
-        assert_allclose(perr_scaled, [ 0.20659803, 0.57204404], rtol=1e-3)
+        for method in ['lm', 'trf', 'dogbox']:
+            popt, pcov = curve_fit(f, xdata, ydata, p0=[2, 0], sigma=sigma,
+                                   method=method)
+            perr_scaled = np.sqrt(np.diag(pcov))
+            assert_allclose(perr_scaled, [0.20659803, 0.57204404], rtol=1e-3)
 
-        popt, pcov = curve_fit(f, xdata, ydata, p0=[2, 0], sigma=3*sigma)
-        perr_scaled = np.sqrt(np.diag(pcov))
-        assert_allclose(perr_scaled, [ 0.20659803, 0.57204404], rtol=1e-3)
+            popt, pcov = curve_fit(f, xdata, ydata, p0=[2, 0], sigma=3*sigma,
+                                   method=method)
+            perr_scaled = np.sqrt(np.diag(pcov))
+            assert_allclose(perr_scaled, [0.20659803, 0.57204404], rtol=1e-3)
 
-        popt, pcov = curve_fit(f, xdata, ydata, p0=[2, 0], sigma=sigma,
-                               absolute_sigma=True)
-        perr = np.sqrt(np.diag(pcov))
-        assert_allclose(perr, [0.30714756, 0.85045308], rtol=1e-3)
+            popt, pcov = curve_fit(f, xdata, ydata, p0=[2, 0], sigma=sigma,
+                                   absolute_sigma=True, method=method)
+            perr = np.sqrt(np.diag(pcov))
+            assert_allclose(perr, [0.30714756, 0.85045308], rtol=1e-3)
 
-        popt, pcov = curve_fit(f, xdata, ydata, p0=[2, 0], sigma=3*sigma,
-                               absolute_sigma=True)
-        perr = np.sqrt(np.diag(pcov))
-        assert_allclose(perr, [3*0.30714756, 3*0.85045308], rtol=1e-3)
+            popt, pcov = curve_fit(f, xdata, ydata, p0=[2, 0], sigma=3*sigma,
+                                   absolute_sigma=True, method=method)
+            perr = np.sqrt(np.diag(pcov))
+            assert_allclose(perr, [3*0.30714756, 3*0.85045308], rtol=1e-3)
 
         # infinite variances
 
         def f_flat(x, a, b):
             return a*x
 
-        popt, pcov = curve_fit(f_flat, xdata, ydata, p0=[2, 0], sigma=sigma)
-        assert_(pcov.shape == (2, 2))
-        pcov_expected = np.array([np.inf]*4).reshape(2, 2)
-        assert_array_equal(pcov, pcov_expected)
+        with warnings.catch_warnings():
+            # suppress warnings when testing with inf's
+            warnings.filterwarnings('ignore', category=OptimizeWarning)
+            popt, pcov = curve_fit(f_flat, xdata, ydata, p0=[2, 0],
+                                   sigma=sigma)
+            assert_(pcov.shape == (2, 2))
+            pcov_expected = np.array([np.inf]*4).reshape(2, 2)
+            assert_array_equal(pcov, pcov_expected)
 
-        popt, pcov = curve_fit(f, xdata[:2], ydata[:2], p0=[2, 0])
-        assert_(pcov.shape == (2, 2))
-        assert_array_equal(pcov, pcov_expected)
+            popt, pcov = curve_fit(f, xdata[:2], ydata[:2], p0=[2, 0])
+            assert_(pcov.shape == (2, 2))
+            assert_array_equal(pcov, pcov_expected)
 
     def test_array_like(self):
         # Test sequence input.  Regression test for gh-3037.
@@ -382,14 +391,90 @@ class TestCurveFit(TestCase):
             return a*x + b
 
         x = [1, 2, 3, 4]
-        y = [2, 4, 6, 8]
-        assert_allclose(curve_fit(f_linear, x, y)[0], [2, 0], atol=1e-10)
+        y = [3, 5, 7, 9]
+        assert_allclose(curve_fit(f_linear, x, y)[0], [2, 1], atol=1e-10)
+
+    def test_indeterminate_covariance(self):
+        # Test that a warning is returned when pcov is indeterminate
+        xdata = np.array([1, 2, 3, 4, 5, 6])
+        ydata = np.array([1, 2, 3, 4, 5.5, 6])
+        _assert_warns(OptimizeWarning, curve_fit,
+                      lambda x, a, b: a*x, xdata, ydata)
+
+    def test_NaN_handling(self):
+        # Test for correct handling of NaNs in input data: gh-3422
+
+        # create input with NaNs
+        xdata = np.array([1, np.nan, 3])
+        ydata = np.array([1, 2, 3])
+
+        assert_raises(ValueError, curve_fit,
+                      lambda x, a, b: a*x + b, xdata, ydata)
+        assert_raises(ValueError, curve_fit,
+                      lambda x, a, b: a*x + b, ydata, xdata)
+
+        assert_raises(ValueError, curve_fit, lambda x, a, b: a*x + b,
+                      xdata, ydata, **{"check_finite": True})
+
+    def test_method_argument(self):
+        def f(x, a, b):
+            return a * np.exp(-b*x)
+
+        xdata = np.linspace(0, 1, 11)
+        ydata = f(xdata, 2., 2.)
+
+        for method in ['trf', 'dogbox', 'lm', None]:
+            popt, pcov = curve_fit(f, xdata, ydata, method=method)
+            assert_allclose(popt, [2., 2.])
+
+        assert_raises(ValueError, curve_fit, f, xdata, ydata, method='unknown')
+
+    def test_bounds(self):
+        def f(x, a, b):
+            return a * np.exp(-b*x)
+
+        xdata = np.linspace(0, 1, 11)
+        ydata = f(xdata, 2., 2.)
+
+        # The minimum w/out bounds is at [2., 2.],
+        # and with bounds it's at [1.5, smth].
+        bounds = ([1., 0], [1.5, 3.])
+        for method in [None, 'trf', 'dogbox']:
+            popt, pcov = curve_fit(f, xdata, ydata, bounds=bounds,
+                                   method=method)
+            assert_allclose(popt[0], 1.5)
+
+        # With bounds, the starting estimate is feasible.
+        popt, pcov = curve_fit(f, xdata, ydata, method='trf',
+                               bounds=([0., 0], [0.6, np.inf]))
+        assert_allclose(popt[0], 0.6)
+
+        # method='lm' doesn't support bounds.
+        assert_raises(ValueError, curve_fit, f, xdata, ydata, bounds=bounds,
+                      method='lm')
+
+    def test_bounds_p0(self):
+        # This test is for issue #5719. The problem was that an initial guess
+        # was ignored when 'trf' or 'dogbox' methods were invoked.
+        def f(x, a):
+            return np.sin(x + a)
+
+        xdata = np.linspace(-2*np.pi, 2*np.pi, 40)
+        ydata = np.sin(xdata)
+        bounds = (-3 * np.pi, 3 * np.pi)
+        for method in ['trf', 'dogbox']:
+            popt_1, _ = curve_fit(f, xdata, ydata, p0=2.1*np.pi)
+            popt_2, _ = curve_fit(f, xdata, ydata, p0=2.1*np.pi,
+                                  bounds=bounds, method=method)
+
+            # If the initial guess is ignored, then popt_2 would be close 0.
+            assert_allclose(popt_1, popt_2)
 
 
 class TestFixedPoint(TestCase):
 
     def test_scalar_trivial(self):
-        """f(x) = 2x; fixed point should be x=0"""
+        # f(x) = 2x; fixed point should be x=0
         def func(x):
             return 2.0*x
         x0 = 1.0
@@ -397,7 +482,7 @@ class TestFixedPoint(TestCase):
         assert_almost_equal(x, 0.0)
 
     def test_scalar_basic1(self):
-        """f(x) = x**2; x0=1.05; fixed point should be x=1"""
+        # f(x) = x**2; x0=1.05; fixed point should be x=1
         def func(x):
             return x**2
         x0 = 1.05
@@ -405,7 +490,7 @@ class TestFixedPoint(TestCase):
         assert_almost_equal(x, 1.0)
 
     def test_scalar_basic2(self):
-        """f(x) = x**0.5; x0=1.05; fixed point should be x=1"""
+        # f(x) = x**0.5; x0=1.05; fixed point should be x=1
         def func(x):
             return x**0.5
         x0 = 1.05
@@ -424,7 +509,7 @@ class TestFixedPoint(TestCase):
         assert_almost_equal(x, [0.0, 0.0])
 
     def test_array_basic1(self):
-        """f(x) = c * x**2; fixed point should be x=1/c"""
+        # f(x) = c * x**2; fixed point should be x=1/c
         def func(x, c):
             return c * x**2
         c = array([0.75, 1.0, 1.25])
@@ -437,7 +522,7 @@ class TestFixedPoint(TestCase):
         assert_almost_equal(x, 1.0/c)
 
     def test_array_basic2(self):
-        """f(x) = c * x**0.5; fixed point should be x=c**2"""
+        # f(x) = c * x**0.5; fixed point should be x=c**2
         def func(x, c):
             return c * x**0.5
         c = array([0.75, 1.0, 1.25])
@@ -445,6 +530,27 @@ class TestFixedPoint(TestCase):
         x = fixed_point(func, x0, args=(c,))
         assert_almost_equal(x, c**2)
 
+    def test_lambertw(self):
+        # python-list/2010-December/594592.html
+        xxroot = fixed_point(lambda xx: np.exp(-2.0*xx)/2.0, 1.0,
+                args=(), xtol=1e-12, maxiter=500)
+        assert_allclose(xxroot, np.exp(-2.0*xxroot)/2.0)
+        assert_allclose(xxroot, lambertw(1)/2)
+
+    def test_no_acceleration(self):
+        # github issue 5460
+        ks = 2
+        kl = 6
+        m = 1.3
+        n0 = 1.001
+        i0 = ((m-1)/m)*(kl/ks/m)**(1/(m-1))
+
+        def func(n):
+            return np.log(kl/ks/n) / np.log((i0*n/(n - 1))) + 1
+
+        n = fixed_point(func, n0, method='iteration')
+        assert_allclose(n, m)
+
 
 if __name__ == "__main__":
     run_module_suite()
