@@ -15,13 +15,38 @@ import warnings
 
 import numpy as np
 from numpy.testing import (assert_raises, assert_allclose, assert_equal,
-                           assert_, TestCase, run_module_suite, dec)
+                           assert_, TestCase, run_module_suite, dec,
+                           assert_almost_equal)
 
+from scipy._lib._testutils import suppressed_stdout
 from scipy import optimize
 
 
-class TestOptimize(object):
-    """ Test case for a simple constrained entropy maximization problem
+def test_check_grad():
+    # Verify if check_grad is able to estimate the derivative of the
+    # logistic function.
+
+    def logit(x):
+        return 1 / (1 + np.exp(-x))
+
+    def der_logit(x):
+        return np.exp(-x) / (1 + np.exp(-x))**2
+
+    x0 = np.array([1.5])
+
+    r = optimize.check_grad(logit, der_logit, x0)
+    assert_almost_equal(r, 0)
+
+    r = optimize.check_grad(logit, der_logit, x0, epsilon=1e-6)
+    assert_almost_equal(r, 0)
+
+    # Check if the epsilon parameter is being considered.
+    r = abs(optimize.check_grad(logit, der_logit, x0, epsilon=1e-1) - 0)
+    assert_(r > 1e-7)
+
+
+class CheckOptimize(object):
+    """ Base test case for a simple constrained entropy maximization problem
     (the machine translation example of Berger et al in
     Computational Linguistics, vol 22, num 1, pp 39--72, 1996.)
     """
@@ -62,22 +87,25 @@ class TestOptimize(object):
     def hessp(self, x, p):
         return np.dot(self.hess(x), p)
 
-    def test_cg(self, use_wrapper=False):
-        """ conjugate gradient optimization routine """
-        if use_wrapper:
-            opts = {'maxiter': self.maxiter, 'disp': False,
+
+class CheckOptimizeParameterized(CheckOptimize):
+
+    @suppressed_stdout
+    def test_cg(self):
+        # conjugate gradient optimization routine
+        if self.use_wrapper:
+            opts = {'maxiter': self.maxiter, 'disp': self.disp,
                     'return_all': False}
             res = optimize.minimize(self.func, self.startparams, args=(),
                                     method='CG', jac=self.grad,
                                     options=opts)
-
             params, fopt, func_calls, grad_calls, warnflag = \
                 res['x'], res['fun'], res['nfev'], res['njev'], res['status']
         else:
-            retval = optimize.fmin_cg(self.func, self.startparams, self.grad, (),
-                                      maxiter=self.maxiter,
-                                      full_output=True, disp=False, retall=False)
-
+            retval = optimize.fmin_cg(self.func, self.startparams,
+                                      self.grad, (), maxiter=self.maxiter,
+                                      full_output=True, disp=self.disp,
+                                      retall=False)
             (params, fopt, func_calls, grad_calls, warnflag) = retval
 
         assert_allclose(self.func(params), self.func(self.solution),
@@ -94,23 +122,24 @@ class TestOptimize(object):
                          [0, -5.05700028e-01, 4.95985862e-01]],
                         atol=1e-14, rtol=1e-7)
 
-    def test_bfgs(self, use_wrapper=False):
-        """ Broyden-Fletcher-Goldfarb-Shanno optimization routine """
-        if use_wrapper:
-            opts = {'maxiter': self.maxiter, 'disp': False,
+    @suppressed_stdout
+    def test_bfgs(self):
+        # Broyden-Fletcher-Goldfarb-Shanno optimization routine
+        if self.use_wrapper:
+            opts = {'maxiter': self.maxiter, 'disp': self.disp,
                     'return_all': False}
             res = optimize.minimize(self.func, self.startparams,
                                     jac=self.grad, method='BFGS', args=(),
                                     options=opts)
 
-            params, fopt, gopt, Hopt, func_calls, grad_calls, warnflag = \
-                    res['x'], res['fun'], res['jac'], res['hess_inv'], \
-                    res['nfev'], res['njev'], res['status']
+            params, fopt, gopt, Hopt, func_calls, grad_calls, warnflag = (
+                    res['x'], res['fun'], res['jac'], res['hess_inv'],
+                    res['nfev'], res['njev'], res['status'])
         else:
             retval = optimize.fmin_bfgs(self.func, self.startparams, self.grad,
                                         args=(), maxiter=self.maxiter,
-                                        full_output=True, disp=False, retall=False)
-
+                                        full_output=True, disp=self.disp,
+                                        retall=False)
             (params, fopt, gopt, Hopt, func_calls, grad_calls, warnflag) = retval
 
         assert_allclose(self.func(params), self.func(self.solution),
@@ -127,72 +156,40 @@ class TestOptimize(object):
                          [0, -5.24885582e-01, 4.87530347e-01]],
                         atol=1e-14, rtol=1e-7)
 
-    def test_bfgs_nan(self):
-        """Test corner case where nan is fed to optimizer.  See #1542."""
-        func = lambda x: x
-        fprime = lambda x: np.ones_like(x)
-        x0 = [np.nan]
-        olderr = np.seterr(over='ignore')
-        try:
-            x = optimize.fmin_bfgs(func, x0, fprime, disp=False)
-            assert_(np.isnan(func(x)))
-        finally:
-            np.seterr(**olderr)
-
-    def test_bfgs_numerical_jacobian(self):
-        """ BFGS with numerical jacobian and a vector epsilon parameter """
-        # define the epsilon parameter using a random vector
-        epsilon = np.sqrt(np.finfo(float).eps) * np.random.rand(len(self.solution))
-
-        params = optimize.fmin_bfgs(self.func, self.startparams,
-                                    epsilon=epsilon, args=(),
-                                    maxiter=self.maxiter, disp=False)
-
-        assert_allclose(self.func(params), self.func(self.solution),
-                        atol=1e-6)
-
-    def test_bfgs_infinite(self, use_wrapper=False):
-        """Test corner case where -Inf is the minimum.  See #1494."""
+    @suppressed_stdout
+    def test_bfgs_infinite(self):
+        # Test corner case where -Inf is the minimum.  See gh-2019.
         func = lambda x: -np.e**-x
         fprime = lambda x: -func(x)
         x0 = [0]
         olderr = np.seterr(over='ignore')
         try:
-            if use_wrapper:
-                opts = {'disp': False}
+            if self.use_wrapper:
+                opts = {'disp': self.disp}
                 x = optimize.minimize(func, x0, jac=fprime, method='BFGS',
                                       args=(), options=opts)['x']
             else:
-                x = optimize.fmin_bfgs(func, x0, fprime, disp=False)
+                x = optimize.fmin_bfgs(func, x0, fprime, disp=self.disp)
             assert_(not np.isfinite(func(x)))
         finally:
             np.seterr(**olderr)
 
-    def test_bfgs_gh_2169(self):
-        def f(x):
-            if x < 0:
-                return 1.79769313e+308
-            else:
-                return x + 1./x
-        xs = optimize.fmin_bfgs(f, [10.], disp=False)
-        assert_allclose(xs, 1.0, rtol=1e-4, atol=1e-4)
-
-    def test_powell(self, use_wrapper=False):
-        """ Powell (direction set) optimization routine
-        """
-        if use_wrapper:
-            opts = {'maxiter': self.maxiter, 'disp': False,
+    @suppressed_stdout
+    def test_powell(self):
+        # Powell (direction set) optimization routine
+        if self.use_wrapper:
+            opts = {'maxiter': self.maxiter, 'disp': self.disp,
                     'return_all': False}
             res = optimize.minimize(self.func, self.startparams, args=(),
                                     method='Powell', options=opts)
-            params, fopt, direc, numiter, func_calls, warnflag = \
-                    res['x'], res['fun'], res['direc'], res['nit'], \
-                    res['nfev'], res['status']
+            params, fopt, direc, numiter, func_calls, warnflag = (
+                    res['x'], res['fun'], res['direc'], res['nit'],
+                    res['nfev'], res['status'])
         else:
             retval = optimize.fmin_powell(self.func, self.startparams,
-                                        args=(), maxiter=self.maxiter,
-                                        full_output=True, disp=False, retall=False)
-
+                                          args=(), maxiter=self.maxiter,
+                                          full_output=True, disp=self.disp,
+                                          retall=False)
             (params, fopt, direc, numiter, func_calls, warnflag) = retval
 
         assert_allclose(self.func(params), self.func(self.solution),
@@ -219,22 +216,22 @@ class TestOptimize(object):
                          [1.72949016, -0.44156936, 0.47576729]],
                         atol=1e-14, rtol=1e-7)
 
-    def test_neldermead(self, use_wrapper=False):
-        """ Nelder-Mead simplex algorithm
-        """
-        if use_wrapper:
-            opts = {'maxiter': self.maxiter, 'disp': False,
+    @suppressed_stdout
+    def test_neldermead(self):
+        # Nelder-Mead simplex algorithm
+        if self.use_wrapper:
+            opts = {'maxiter': self.maxiter, 'disp': self.disp,
                     'return_all': False}
             res = optimize.minimize(self.func, self.startparams, args=(),
                                     method='Nelder-mead', options=opts)
-            params, fopt, numiter, func_calls, warnflag = \
-                    res['x'], res['fun'], res['nit'], res['nfev'], \
-                    res['status']
+            params, fopt, numiter, func_calls, warnflag, final_simplex = (
+                    res['x'], res['fun'], res['nit'], res['nfev'],
+                    res['status'], res['final_simplex'])
         else:
             retval = optimize.fmin(self.func, self.startparams,
                                         args=(), maxiter=self.maxiter,
-                                        full_output=True, disp=False, retall=False)
-
+                                        full_output=True, disp=self.disp,
+                                        retall=False)
             (params, fopt, numiter, func_calls, warnflag) = retval
 
         assert_allclose(self.func(params), self.func(self.solution),
@@ -251,11 +248,11 @@ class TestOptimize(object):
                          [0.19572515, -0.63648426, 0.35838135]],
                         atol=1e-14, rtol=1e-7)
 
-    def test_ncg(self, use_wrapper=False):
-        """ line-search Newton conjugate gradient optimization routine
-        """
-        if use_wrapper:
-            opts = {'maxiter': self.maxiter, 'disp': False,
+    @suppressed_stdout
+    def test_ncg(self):
+        # line-search Newton conjugate gradient optimization routine
+        if self.use_wrapper:
+            opts = {'maxiter': self.maxiter, 'disp': self.disp,
                     'return_all': False}
             retval = optimize.minimize(self.func, self.startparams,
                                        method='Newton-CG', jac=self.grad,
@@ -263,7 +260,7 @@ class TestOptimize(object):
         else:
             retval = optimize.fmin_ncg(self.func, self.startparams, self.grad,
                                        args=(), maxiter=self.maxiter,
-                                       full_output=False, disp=False,
+                                       full_output=False, disp=self.disp,
                                        retall=False)
 
         params = retval
@@ -285,10 +282,11 @@ class TestOptimize(object):
                          [-4.35700753e-07, -5.24869401e-01, 4.87527774e-01]],
                         atol=1e-6, rtol=1e-7)
 
-    def test_ncg_hess(self, use_wrapper=False):
-        """ Newton conjugate gradient with Hessian """
-        if use_wrapper:
-            opts = {'maxiter': self.maxiter, 'disp': False,
+    @suppressed_stdout
+    def test_ncg_hess(self):
+        # Newton conjugate gradient with Hessian
+        if self.use_wrapper:
+            opts = {'maxiter': self.maxiter, 'disp': self.disp,
                     'return_all': False}
             retval = optimize.minimize(self.func, self.startparams,
                                        method='Newton-CG', jac=self.grad,
@@ -298,7 +296,7 @@ class TestOptimize(object):
             retval = optimize.fmin_ncg(self.func, self.startparams, self.grad,
                                        fhess=self.hess,
                                        args=(), maxiter=self.maxiter,
-                                       full_output=False, disp=False,
+                                       full_output=False, disp=self.disp,
                                        retall=False)
 
         params = retval
@@ -319,10 +317,11 @@ class TestOptimize(object):
                          [-4.35700753e-07, -5.24869401e-01, 4.87527774e-01]],
                         atol=1e-6, rtol=1e-7)
 
-    def test_ncg_hessp(self, use_wrapper=False):
-        """ Newton conjugate gradient with Hessian times a vector p """
-        if use_wrapper:
-            opts = {'maxiter': self.maxiter, 'disp': False,
+    @suppressed_stdout
+    def test_ncg_hessp(self):
+        # Newton conjugate gradient with Hessian times a vector p.
+        if self.use_wrapper:
+            opts = {'maxiter': self.maxiter, 'disp': self.disp,
                     'return_all': False}
             retval = optimize.minimize(self.func, self.startparams,
                                        method='Newton-CG', jac=self.grad,
@@ -332,7 +331,7 @@ class TestOptimize(object):
             retval = optimize.fmin_ncg(self.func, self.startparams, self.grad,
                                        fhess_p=self.hessp,
                                        args=(), maxiter=self.maxiter,
-                                       full_output=False, disp=False,
+                                       full_output=False, disp=self.disp,
                                        retall=False)
 
         params = retval
@@ -353,9 +352,81 @@ class TestOptimize(object):
                          [-4.35700753e-07, -5.24869401e-01, 4.87527774e-01]],
                         atol=1e-6, rtol=1e-7)
 
+
+class TestOptimizeWrapperDisp(CheckOptimizeParameterized):
+    use_wrapper = True
+    disp = True
+
+
+class TestOptimizeWrapperNoDisp(CheckOptimizeParameterized):
+    use_wrapper = True
+    disp = False
+
+
+class TestOptimizeNoWrapperDisp(CheckOptimizeParameterized):
+    use_wrapper = False
+    disp = True
+
+
+class TestOptimizeNoWrapperNoDisp(CheckOptimizeParameterized):
+    use_wrapper = False
+    disp = False
+
+
+class TestOptimizeSimple(CheckOptimize):
+
+    def test_bfgs_nan(self):
+        # Test corner case where nan is fed to optimizer.  See gh-2067.
+        func = lambda x: x
+        fprime = lambda x: np.ones_like(x)
+        x0 = [np.nan]
+        with np.errstate(over='ignore', invalid='ignore'):
+            x = optimize.fmin_bfgs(func, x0, fprime, disp=False)
+            assert_(np.isnan(func(x)))
+
+    def test_bfgs_nan_return(self):
+        # Test corner cases where fun returns NaN. See gh-4793.
+
+        # First case: NaN from first call.
+        func = lambda x: np.nan
+        with np.errstate(invalid='ignore'):
+            result = optimize.minimize(func, 0)
+
+        assert_(np.isnan(result['fun']))
+        assert_(result['success'] is False)
+
+        # Second case: NaN from second call.
+        func = lambda x: 0 if x == 0 else np.nan
+        fprime = lambda x: np.ones_like(x)  # Steer away from zero.
+        with np.errstate(invalid='ignore'):
+            result = optimize.minimize(func, 0, jac=fprime)
+
+        assert_(np.isnan(result['fun']))
+        assert_(result['success'] is False)
+
+    def test_bfgs_numerical_jacobian(self):
+        # BFGS with numerical jacobian and a vector epsilon parameter.
+        # define the epsilon parameter using a random vector
+        epsilon = np.sqrt(np.finfo(float).eps) * np.random.rand(len(self.solution))
+
+        params = optimize.fmin_bfgs(self.func, self.startparams,
+                                    epsilon=epsilon, args=(),
+                                    maxiter=self.maxiter, disp=False)
+
+        assert_allclose(self.func(params), self.func(self.solution),
+                        atol=1e-6)
+
+    def test_bfgs_gh_2169(self):
+        def f(x):
+            if x < 0:
+                return 1.79769313e+308
+            else:
+                return x + 1./x
+        xs = optimize.fmin_bfgs(f, [10.], disp=False)
+        assert_allclose(xs, 1.0, rtol=1e-4, atol=1e-4)
+
     def test_l_bfgs_b(self):
-        """ limited-memory bound-constrained BFGS algorithm
-        """
+        # limited-memory bound-constrained BFGS algorithm
         retval = optimize.fmin_l_bfgs_b(self.func, self.startparams,
                                         self.grad, args=(),
                                         maxiter=self.maxiter)
@@ -377,7 +448,7 @@ class TestOptimize(object):
                         atol=1e-14, rtol=1e-7)
 
     def test_l_bfgs_b_numjac(self):
-        """ L-BFGS-B with numerical jacobian """
+        # L-BFGS-B with numerical jacobian
         retval = optimize.fmin_l_bfgs_b(self.func, self.startparams,
                                         approx_grad=True,
                                         maxiter=self.maxiter)
@@ -388,7 +459,7 @@ class TestOptimize(object):
                         atol=1e-6)
 
     def test_l_bfgs_b_funjac(self):
-        """ L-BFGS-B with combined objective function and jacobian """
+        # L-BFGS-B with combined objective function and jacobian
         def fun(x):
             return self.func(x), self.grad(x)
 
@@ -401,7 +472,7 @@ class TestOptimize(object):
                         atol=1e-6)
 
     def test_minimize_l_bfgs_b(self):
-        """ Minimize with L-BFGS-B method """
+        # Minimize with L-BFGS-B method
         opts = {'disp': False, 'maxiter': self.maxiter}
         r = optimize.minimize(self.func, self.startparams,
                               method='L-BFGS-B', jac=self.grad,
@@ -433,6 +504,13 @@ class TestOptimize(object):
 
             assert_allclose(v, self.func(self.solution), rtol=tol)
 
+    def test_minimize_l_bfgs_maxls(self):
+        # check that the maxls is passed down to the Fortran routine
+        sol = optimize.minimize(optimize.rosen, np.array([-1.2,1.0]),
+                                method='L-BFGS-B', jac=optimize.rosen_der,
+                                options={'disp': False, 'maxls': 1})
+        assert_(not sol.success)
+
     def test_custom(self):
         # This function comes from the documentation example.
         def custmin(fun, x0, args=(), maxfev=None, stepsize=0.1,
@@ -471,27 +549,6 @@ class TestOptimize(object):
                                 options=dict(stepsize=0.05))
         assert_allclose(res.x, 1.0, rtol=1e-4, atol=1e-4)
 
-    def test_minimize(self):
-        """Tests for the minimize wrapper."""
-        self.setUp()
-        self.test_bfgs(True)
-        self.setUp()
-        self.test_bfgs_infinite(True)
-        self.setUp()
-        self.test_cg(True)
-        self.setUp()
-        self.test_ncg(True)
-        self.setUp()
-        self.test_ncg_hess(True)
-        self.setUp()
-        self.test_ncg_hessp(True)
-        self.setUp()
-        self.test_neldermead(True)
-        self.setUp()
-        self.test_powell(True)
-        self.setUp()
-        self.test_custom()
-
     def test_minimize_tol_parameter(self):
         # Check that the minimize() tol= argument does something
         def func(z):
@@ -503,22 +560,19 @@ class TestOptimize(object):
             return np.array([2*x*y**2 + 4*x**3, 2*x**2*y])
 
         for method in ['nelder-mead', 'powell', 'cg', 'bfgs',
-                       'newton-cg', 'anneal', 'l-bfgs-b', 'tnc',
+                       'newton-cg', 'l-bfgs-b', 'tnc',
                        'cobyla', 'slsqp']:
-            if method in ('nelder-mead', 'powell', 'anneal', 'cobyla'):
+            if method in ('nelder-mead', 'powell', 'cobyla'):
                 jac = None
             else:
                 jac = dfunc
 
-            with warnings.catch_warnings():
-                # suppress deprecation warning for 'anneal'
-                warnings.filterwarnings('ignore', category=DeprecationWarning)
-                sol1 = optimize.minimize(func, [1,1], jac=jac, tol=1e-10,
-                                         method=method)
-                sol2 = optimize.minimize(func, [1,1], jac=jac, tol=1.0,
-                                         method=method)
-                assert_(func(sol1.x) < func(sol2.x),
-                        "%s: %s vs. %s" % (method, func(sol1.x), func(sol2.x)))
+            sol1 = optimize.minimize(func, [1, 1], jac=jac, tol=1e-10,
+                                     method=method)
+            sol2 = optimize.minimize(func, [1, 1], jac=jac, tol=1.0,
+                                     method=method)
+            assert_(func(sol1.x) < func(sol2.x),
+                    "%s: %s vs. %s" % (method, func(sol1.x), func(sol2.x)))
 
     def test_no_increase(self):
         # Check that the solver doesn't return a value worse than the
@@ -536,7 +590,7 @@ class TestOptimize(object):
             x0 = np.array([2.0])
             f0 = func(x0)
             jac = bad_grad
-            if method in ['nelder-mead', 'powell', 'anneal', 'cobyla']:
+            if method in ['nelder-mead', 'powell', 'cobyla']:
                 jac = None
             sol = optimize.minimize(func, x0, jac=jac, method=method,
                                     options=dict(maxiter=20))
@@ -550,17 +604,15 @@ class TestOptimize(object):
                        'cobyla', 'slsqp']:
             yield check, method
 
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            yield check, 'anneal'
-
     def test_slsqp_respect_bounds(self):
-        # github issue 3108
+        # Regression test for gh-3108
         def f(x):
             return sum((x - np.array([1., 2., 3., 4.]))**2)
+
         def cons(x):
             a = np.array([[-1, -1, -1, -1], [-3, -3, -2, -1]])
             return np.concatenate([np.dot(a, x) + np.array([5, 10]), x])
+
         x0 = np.array([0.5, 1., 1.5, 2.])
         res = optimize.minimize(f, x0, method='slsqp',
                                 constraints={'type': 'ineq', 'fun': cons})
@@ -569,6 +621,7 @@ class TestOptimize(object):
     def test_minimize_automethod(self):
         def f(x):
             return x**2
+
         def cons(x):
             return x - 2
 
@@ -586,9 +639,20 @@ class TestOptimize(object):
         assert_allclose(sol_3.x, 5, atol=1e-8)
         assert_allclose(sol_4.x, 2, atol=1e-8)
 
+    def test_minimize_coerce_args_param(self):
+        # Regression test for gh-3503
+        def Y(x, c):
+            return np.sum((x-c)**2)
+
+        def dY_dx(x, c=None):
+            return 2*(x-c)
+
+        c = np.array([3, 1, 4, 1, 5, 9, 2, 6, 5, 3, 5])
+        xinit = np.random.randn(len(c))
+        optimize.minimize(Y, xinit, jac=dY_dx, args=(c), method="BFGS")
+
 
 class TestLBFGSBBounds(TestCase):
-    """ Tests for L-BFGS-B with bounds """
     def setUp(self):
         self.bounds = ((1, None), (None, None))
         self.solution = (1, 0)
@@ -603,7 +667,6 @@ class TestLBFGSBBounds(TestCase):
         return self.fun(x, p), self.jac(x, p)
 
     def test_l_bfgs_b_bounds(self):
-        """ L-BFGS-B with bounds """
         x, f, d = optimize.fmin_l_bfgs_b(self.fun, [0, -1],
                                          fprime=self.jac,
                                          bounds=self.bounds)
@@ -611,14 +674,14 @@ class TestLBFGSBBounds(TestCase):
         assert_allclose(x, self.solution, atol=1e-6)
 
     def test_l_bfgs_b_funjac(self):
-        """ L-BFGS-B with fun and jac combined and extra arguments """
+        # L-BFGS-B with fun and jac combined and extra arguments
         x, f, d = optimize.fmin_l_bfgs_b(self.fj, [0, -1], args=(2.0, ),
                                          bounds=self.bounds)
         assert_(d['warnflag'] == 0, d['task'])
         assert_allclose(x, self.solution, atol=1e-6)
 
     def test_minimize_l_bfgs_b_bounds(self):
-        """ Minimize with method='L-BFGS-B' with bounds """
+        # Minimize with method='L-BFGS-B' with bounds
         res = optimize.minimize(self.fun, [0, -1], method='L-BFGS-B',
                                 jac=self.jac, bounds=self.bounds)
         assert_(res['success'], res['message'])
@@ -626,7 +689,6 @@ class TestLBFGSBBounds(TestCase):
 
 
 class TestOptimizeScalar(TestCase):
-    """Tests for scalar optimizers"""
     def setUp(self):
         self.solution = 1.5
 
@@ -635,7 +697,6 @@ class TestOptimizeScalar(TestCase):
         return (x - a)**2 - 0.8
 
     def test_brent(self):
-        """ brent algorithm """
         x = optimize.brent(self.fun)
         assert_allclose(x, self.solution, atol=1e-6)
 
@@ -649,7 +710,6 @@ class TestOptimizeScalar(TestCase):
         assert_allclose(x, self.solution, atol=1e-6)
 
     def test_golden(self):
-        """ golden algorithm """
         x = optimize.golden(self.fun)
         assert_allclose(x, self.solution, atol=1e-6)
 
@@ -663,7 +723,6 @@ class TestOptimizeScalar(TestCase):
         assert_allclose(x, self.solution, atol=1e-6)
 
     def test_fminbound(self):
-        """Test fminbound """
         x = optimize.fminbound(self.fun, 0, 1)
         assert_allclose(x, 1, atol=1e-4)
 
@@ -675,8 +734,11 @@ class TestOptimizeScalar(TestCase):
         assert_raises(ValueError, optimize.fminbound, self.fun, 5, 1)
 
     def test_fminbound_scalar(self):
-        assert_raises(ValueError, optimize.fminbound, self.fun,
-                      np.zeros(2), 1)
+        try:
+            optimize.fminbound(self.fun, np.zeros((1, 2)), 1)
+            self.fail("exception not raised")
+        except ValueError as e:
+            assert_('must be scalar' in str(e))
 
         x = optimize.fminbound(self.fun, 1, np.array(5))
         assert_allclose(x, self.solution, atol=1e-6)
@@ -686,6 +748,13 @@ class TestOptimizeScalar(TestCase):
         x = optimize.minimize_scalar(self.fun).x
         assert_allclose(x, self.solution, atol=1e-6)
 
+        x = optimize.minimize_scalar(self.fun, method='Brent')
+        assert_(x.success)
+
+        x = optimize.minimize_scalar(self.fun, method='Brent',
+                                     options=dict(maxiter=3))
+        assert_(not x.success)
+
         x = optimize.minimize_scalar(self.fun, bracket=(-3, -2),
                                     args=(1.5, ), method='Brent').x
         assert_allclose(x, self.solution, atol=1e-6)
@@ -768,6 +837,14 @@ class TestOptimizeScalar(TestCase):
                                        options=dict(stepsize=0.05))
         assert_allclose(res.x, self.solution, atol=1e-6)
 
+    def test_minimize_scalar_coerce_args_param(self):
+        # Regression test for gh-3503
+        optimize.minimize_scalar(self.fun, args=1.5)
+
+
+def test_brent_negative_tolerance():
+    assert_raises(ValueError, optimize.brent, np.cos, tol=-.01)
+
 
 class TestNewtonCg(object):
     def test_rosenbrock(self):
@@ -796,7 +873,7 @@ class TestNewtonCg(object):
 class TestRosen(TestCase):
 
     def test_hess(self):
-        """Compare rosen_hess(x) times p with rosen_hess_prod(x,p) (ticket #1248)"""
+        # Compare rosen_hess(x) times p with rosen_hess_prod(x,p).  See gh-1775
         x = np.array([3, 4, 5])
         p = np.array([2, 2, 2])
         hp = optimize.rosen_hess_prod(x, p)
@@ -830,5 +907,101 @@ himmelblau_x0 = [-0.27, -0.9]
 himmelblau_xopt = [3, 2]
 himmelblau_min = 0.0
 
+
+def test_minimize_multiple_constraints():
+    # Regression test for gh-4240.
+    def func(x):
+        return np.array([25 - 0.2 * x[0] - 0.4 * x[1] - 0.33 * x[2]])
+
+    def func1(x):
+        return np.array([x[1]])
+
+    def func2(x):
+        return np.array([x[2]])
+
+    cons = ({'type': 'ineq', 'fun': func},
+            {'type': 'ineq', 'fun': func1},
+            {'type': 'ineq', 'fun': func2})
+
+    f = lambda x: -1 * (x[0] + x[1] + x[2])
+
+    res = optimize.minimize(f, [0, 0, 0], method='SLSQP', constraints=cons)
+    assert_allclose(res.x, [125, 0, 0], atol=1e-10)
+
+
+class TestOptimizeResultAttributes(TestCase):
+    # Test that all minimizers return an OptimizeResult containing
+    # all the OptimizeResult attributes
+    def setUp(self):
+        self.x0 = [5, 5]
+        self.func = optimize.rosen
+        self.jac = optimize.rosen_der
+        self.hess = optimize.rosen_hess
+        self.hessp = optimize.rosen_hess_prod
+        self.bounds = [(0., 10.), (0., 10.)]
+
+    def test_attributes_present(self):
+        methods = ['Nelder-Mead', 'Powell', 'CG', 'BFGS', 'Newton-CG',
+                   'L-BFGS-B', 'TNC', 'COBYLA', 'SLSQP', 'dogleg',
+                   'trust-ncg']
+        attributes = ['nit', 'nfev', 'x', 'success', 'status', 'fun',
+                      'message']
+        skip = {'COBYLA': ['nit']}
+        for method in methods:
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
+                res = optimize.minimize(self.func, self.x0, method=method,
+                                        jac=self.jac, hess=self.hess,
+                                        hessp=self.hessp)
+            for attribute in attributes:
+                if method in skip and attribute in skip[method]:
+                    continue
+
+                assert_(hasattr(res, attribute))
+
+
+class TestBrute:
+    # Test the "brute force" method
+    def setUp(self):
+        self.params = (2, 3, 7, 8, 9, 10, 44, -1, 2, 26, 1, -2, 0.5)
+        self.rranges = (slice(-4, 4, 0.25), slice(-4, 4, 0.25))
+        self.solution = np.array([-1.05665192, 1.80834843])
+
+    def f1(self, z, *params):
+        x, y = z
+        a, b, c, d, e, f, g, h, i, j, k, l, scale = params
+        return (a * x**2 + b * x * y + c * y**2 + d*x + e*y + f)
+
+    def f2(self, z, *params):
+        x, y = z
+        a, b, c, d, e, f, g, h, i, j, k, l, scale = params
+        return (-g*np.exp(-((x-h)**2 + (y-i)**2) / scale))
+
+    def f3(self, z, *params):
+        x, y = z
+        a, b, c, d, e, f, g, h, i, j, k, l, scale = params
+        return (-j*np.exp(-((x-k)**2 + (y-l)**2) / scale))
+
+    def func(self, z, *params):
+        return self.f1(z, *params) + self.f2(z, *params) + self.f3(z, *params)
+
+    @suppressed_stdout
+    def test_brute(self):
+        # test fmin
+        resbrute = optimize.brute(self.func, self.rranges, args=self.params,
+                                  full_output=True, finish=optimize.fmin)
+        assert_allclose(resbrute[0], self.solution, atol=1e-3)
+        assert_allclose(resbrute[1], self.func(self.solution, *self.params),
+                        atol=1e-3)
+
+        # test minimize
+        resbrute = optimize.brute(self.func, self.rranges, args=self.params,
+                                  full_output=True,
+                                  finish=optimize.minimize)
+        assert_allclose(resbrute[0], self.solution, atol=1e-3)
+        assert_allclose(resbrute[1], self.func(self.solution, *self.params),
+                        atol=1e-3)
+
+
 if __name__ == "__main__":
     run_module_suite()
