@@ -12,19 +12,19 @@ import math
 import warnings
 
 import numpy as np
-from numpy import array, eye, dot, sqrt, double, exp, random
+from numpy import array, eye, exp, random
 from numpy.linalg import matrix_power
 from numpy.testing import (TestCase, run_module_suite,
         assert_allclose, assert_, assert_array_almost_equal, assert_equal,
-        assert_array_equal, assert_array_almost_equal_nulp, decorators)
+        assert_array_almost_equal_nulp)
 
 from scipy.sparse import csc_matrix, SparseEfficiencyWarning
 from scipy.sparse.construct import eye as speye
-from scipy.sparse.linalg.matfuncs import (expm,
+from scipy.sparse.linalg.matfuncs import (expm, _expm,
         ProductOperator, MatrixPowerOperator,
         _onenorm_matrix_power_nnm)
 from scipy.linalg import logm
-from scipy.misc import factorial
+from scipy.special import factorial
 import scipy.sparse
 import scipy.sparse.linalg
 
@@ -136,7 +136,10 @@ class TestExpM(TestCase):
             for scale in [1e-2, 1e-1, 5e-1, 1, 10]:
                 a = scale * speye(3, 3, dtype=dtype, format='csc')
                 e = exp(scale) * eye(3, dtype=dtype)
-                assert_array_almost_equal_nulp(expm(a).toarray(), e, nulp=100)
+                exact_onenorm = _expm(a, use_exact_onenorm=True).toarray()
+                inexact_onenorm = _expm(a, use_exact_onenorm=False).toarray()
+                assert_array_almost_equal_nulp(exact_onenorm, e, nulp=100)
+                assert_array_almost_equal_nulp(inexact_onenorm, e, nulp=100)
 
     def test_padecases_dtype_sparse_complex(self):
         # float32 and complex64 lead to errors in spsolve/UMFpack
@@ -159,6 +162,14 @@ class TestExpM(TestCase):
                         A = A + 1j * random.rand(n, n) * scale
                     assert_array_almost_equal(expm(logm(A)), A)
 
+    def test_integer_matrix(self):
+        Q = np.array([
+            [-3, 1, 1, 1],
+            [1, -3, 1, 1],
+            [1, 1, -3, 1],
+            [1, 1, 1, -3]])
+        assert_allclose(expm(Q), expm(1.0 * Q))
+
     def test_triangularity_perturbation(self):
         # Experiment (1) of
         # Awad H. Al-Mohy and Nicholas J. Higham (2012)
@@ -186,7 +197,6 @@ class TestExpM(TestCase):
         # so that it becomes technically not upper triangular.
         random.seed(1234)
         tiny = 1e-17
-        n = 4
         A_logm_perturbed = A_logm.copy()
         A_logm_perturbed[1, 0] = tiny
         A_expm_logm_perturbed = expm(A_logm_perturbed)
