@@ -111,6 +111,8 @@ call_thunk(char ret_spec, const char *spec, thunk_t *thunk, PyObject *args)
     int j, k, arg_j;
     const char *p;
     Py_ssize_t ret;
+    Py_ssize_t max_array_size = 0;
+    NPY_BEGIN_THREADS_DEF;
 
     if (!PyTuple_Check(args)) {
         PyErr_SetString(PyExc_ValueError, "args is not a tuple");
@@ -214,14 +216,18 @@ call_thunk(char ret_spec, const char *spec, thunk_t *thunk, PyObject *args)
         /* Find a compatible supported data type */
         dtype = PyArray_DESCR(arg_arrays[j]);
         for (k = 0; k < n_supported_typenums; ++k) {
-            if (PyArray_CanCastSafely(dtype->type_num, supported_typenums[k]))
+            if (PyArray_CanCastSafely(dtype->type_num, supported_typenums[k]) &&
+                (cur_typenum == -1 || PyArray_CanCastSafely(cur_typenum, supported_typenums[k])))
             {
-                if (cur_typenum == -1 || !PyArray_CanCastSafely(supported_typenums[k], cur_typenum)) {
-                    cur_typenum = supported_typenums[k];
-                }
+                cur_typenum = supported_typenums[k];
                 break;
             }
         }
+        if (k == n_supported_typenums) {
+            PyErr_SetString(PyExc_ValueError,
+                            "unsupported data types in input");
+            goto fail;
+        }
 
         if (*p == 'I') {
             I_typenum = cur_typenum;
@@ -233,13 +239,13 @@ call_thunk(char ret_spec, const char *spec, thunk_t *thunk, PyObject *args)
 
     if (arg_j != PyTuple_Size(args)) {
         PyErr_SetString(PyExc_ValueError, "too many arguments");
-        return NULL;
+        goto fail;
     }
 
     if ((I_in_arglist && I_typenum == -1) ||
         (T_in_arglist && T_typenum == -1)) {
-        PyErr_SetString(PyExc_RuntimeError,
-                        "internal error: failed to resolve data types");
+        PyErr_SetString(PyExc_ValueError,
+                        "unsupported data types in input");
         goto fail;
     }
 
@@ -270,7 +276,8 @@ call_thunk(char ret_spec, const char *spec, thunk_t *thunk, PyObject *args)
                 arg_list[j] = std::malloc(sizeof(npy_int64));
                 *(npy_int64*)arg_list[j] = (npy_int64)value;
             }
-            else if (value == (npy_int32)value) {
+            else if (PyArray_EquivTypenums(I_typenum, NPY_INT32)
+                     && value == (npy_int32)value) {
                 arg_list[j] = std::malloc(sizeof(npy_int32));
                 *(npy_int32*)arg_list[j] = (npy_int32)value;
             }
@@ -286,10 +293,16 @@ call_thunk(char ret_spec, const char *spec, thunk_t *thunk, PyObject *args)
         }
         else if (*p == 'V') {
             arg_list[j] = allocate_std_vector_typenum(I_typenum);
+            if (arg_list[j] == NULL) {
+                goto fail;
+            }
             continue;
         }
         else if (*p == 'W') {
             arg_list[j] = allocate_std_vector_typenum(T_typenum);
+            if (arg_list[j] == NULL) {
+                goto fail;
+            }
             continue;
         }
         else {
@@ -297,31 +310,51 @@ call_thunk(char ret_spec, const char *spec, thunk_t *thunk, PyObject *args)
 
             /* Cast if necessary */
             arg = arg_arrays[j];
-            if (!PyArray_EquivTypenums(PyArray_DESCR(arg)->type_num,
-                                       cur_typenum))
-            {
+            if (PyArray_EquivTypenums(PyArray_DESCR(arg)->type_num, cur_typenum)) {
+                /* No cast needed. */
+            }
+            else if (!is_output[j] || PyArray_CanCastSafely(cur_typenum, PyArray_DESCR(arg)->type_num)) {
+                /* Cast needed. Output arrays require safe cast back. */
                 arg_arrays[j] = c_array_from_object(arg, cur_typenum, is_output[j]);
                 Py_DECREF(arg);
                 if (arg_arrays[j] == NULL) {
                     goto fail;
                 }
             }
+            else {
+                /* Cast back into output array was not safe. */
+                PyErr_SetString(PyExc_ValueError,
+                                "Output dtype not compatible with inputs.");
+                goto fail;
+            }
         }
 
         /* Grab value */
         arg_list[j] = PyArray_DATA(arg_arrays[j]);
+
+        /* Find maximum array size */
+        if (PyArray_SIZE(arg_arrays[j]) > max_array_size) {
+            max_array_size = PyArray_SIZE(arg_arrays[j]);
+        }
     }
 
 
     /*
      * Call thunk
      */
+    if (max_array_size > 100) {
+        /* Threshold GIL release: it's not a free operation */
+        NPY_BEGIN_THREADS;
+    }
     try {
         ret = thunk(I_typenum, T_typenum, arg_list);
+        NPY_END_THREADS;
     } catch (const std::bad_alloc &e) {
+        NPY_END_THREADS;
         PyErr_SetString(PyExc_MemoryError, e.what());
         goto fail;
     } catch (const std::exception &e) {
+        NPY_END_THREADS;
         PyErr_SetString(PyExc_RuntimeError, e.what());
         goto fail;
     }
@@ -424,20 +457,27 @@ static void *allocate_std_vector_typenum(int typenum)
         return (void*)(new std::vector<ctype>());               \
     }
 
-    PROCESS(NPY_BOOL, npy_bool_wrapper);
-    PROCESS(NPY_BYTE, npy_byte);
-    PROCESS(NPY_UBYTE, npy_ubyte);
-    PROCESS(NPY_SHORT, npy_short);
-    PROCESS(NPY_USHORT, npy_ushort);
-    PROCESS(NPY_INT, npy_uint);
-    PROCESS(NPY_LONG, npy_ulong);
-    PROCESS(NPY_LONGLONG, npy_ulonglong);
-    PROCESS(NPY_FLOAT, npy_float);
-    PROCESS(NPY_DOUBLE, npy_double);
-    PROCESS(NPY_LONGDOUBLE, npy_longdouble);
-    PROCESS(NPY_CFLOAT, npy_cfloat_wrapper);
-    PROCESS(NPY_CDOUBLE, npy_cdouble_wrapper);
-    PROCESS(NPY_CLONGDOUBLE, npy_clongdouble_wrapper);
+    try {
+        PROCESS(NPY_BOOL, npy_bool_wrapper);
+        PROCESS(NPY_BYTE, npy_byte);
+        PROCESS(NPY_UBYTE, npy_ubyte);
+        PROCESS(NPY_SHORT, npy_short);
+        PROCESS(NPY_USHORT, npy_ushort);
+        PROCESS(NPY_INT, npy_int);
+        PROCESS(NPY_UINT, npy_uint);
+        PROCESS(NPY_LONG, npy_long);
+        PROCESS(NPY_ULONG, npy_ulong);
+        PROCESS(NPY_LONGLONG, npy_longlong);
+        PROCESS(NPY_ULONGLONG, npy_ulonglong);
+        PROCESS(NPY_FLOAT, npy_float);
+        PROCESS(NPY_DOUBLE, npy_double);
+        PROCESS(NPY_LONGDOUBLE, npy_longdouble);
+        PROCESS(NPY_CFLOAT, npy_cfloat_wrapper);
+        PROCESS(NPY_CDOUBLE, npy_cdouble_wrapper);
+        PROCESS(NPY_CLONGDOUBLE, npy_clongdouble_wrapper);
+    } catch (std::exception &e) {
+        /* failed */
+    }
 
 #undef PROCESS
 
@@ -458,9 +498,12 @@ static void free_std_vector_typenum(int typenum, void *p)
     PROCESS(NPY_UBYTE, npy_ubyte);
     PROCESS(NPY_SHORT, npy_short);
     PROCESS(NPY_USHORT, npy_ushort);
-    PROCESS(NPY_INT, npy_uint);
-    PROCESS(NPY_LONG, npy_ulong);
-    PROCESS(NPY_LONGLONG, npy_ulonglong);
+    PROCESS(NPY_INT, npy_int);
+    PROCESS(NPY_UINT, npy_uint);
+    PROCESS(NPY_LONG, npy_long);
+    PROCESS(NPY_ULONG, npy_ulong);
+    PROCESS(NPY_LONGLONG, npy_longlong);
+    PROCESS(NPY_ULONGLONG, npy_ulonglong);
     PROCESS(NPY_FLOAT, npy_float);
     PROCESS(NPY_DOUBLE, npy_double);
     PROCESS(NPY_LONGDOUBLE, npy_longdouble);
@@ -491,9 +534,12 @@ static PyObject *array_from_std_vector_and_free(int typenum, void *p)
     PROCESS(NPY_UBYTE, npy_ubyte);
     PROCESS(NPY_SHORT, npy_short);
     PROCESS(NPY_USHORT, npy_ushort);
-    PROCESS(NPY_INT, npy_uint);
-    PROCESS(NPY_LONG, npy_ulong);
-    PROCESS(NPY_LONGLONG, npy_ulonglong);
+    PROCESS(NPY_INT, npy_int);
+    PROCESS(NPY_UINT, npy_uint);
+    PROCESS(NPY_LONG, npy_long);
+    PROCESS(NPY_ULONG, npy_ulong);
+    PROCESS(NPY_LONGLONG, npy_longlong);
+    PROCESS(NPY_ULONGLONG, npy_ulonglong);
     PROCESS(NPY_FLOAT, npy_float);
     PROCESS(NPY_DOUBLE, npy_double);
     PROCESS(NPY_LONGDOUBLE, npy_longdouble);
