@@ -4,13 +4,13 @@ import os
 import copy
 
 import numpy as np
-from numpy.testing import assert_equal, assert_almost_equal, run_module_suite,\
-     assert_, dec, assert_allclose, assert_array_equal, assert_raises
-from scipy.lib.six import xrange
+from numpy.testing import (assert_equal, assert_almost_equal, run_module_suite,
+                           assert_, dec, assert_allclose, assert_array_equal,
+                           assert_raises)
+from scipy._lib.six import xrange
 
 import scipy.spatial.qhull as qhull
 from scipy.spatial import cKDTree as KDTree
-from scipy.lib._version import NumpyVersion
 
 
 def sorted_tuple(x):
@@ -230,6 +230,37 @@ class TestUtilities(object):
 
         assert_equal(tri.convex_hull, [[3, 2], [1, 2], [1, 0], [3, 0]])
 
+    def test_volume_area(self):
+        #Basic check that we get back the correct volume and area for a cube
+        points = np.array([(0, 0, 0), (0, 1, 0), (1, 0, 0), (1, 1, 0),
+                           (0, 0, 1), (0, 1, 1), (1, 0, 1), (1, 1, 1)])
+        hull = qhull.ConvexHull(points)
+
+        assert_allclose(hull.volume, 1., rtol=1e-14,
+                        err_msg="Volume of cube is incorrect")
+        assert_allclose(hull.area, 6., rtol=1e-14,
+                        err_msg="Area of cube is incorrect")
+
+    def test_random_volume_area(self):
+        #Test that the results for a random 10-point convex are
+        #coherent with the output of qconvex Qt s FA
+        points = np.array([(0.362568364506, 0.472712355305, 0.347003084477),
+                           (0.733731893414, 0.634480295684, 0.950513180209),
+                           (0.511239955611, 0.876839441267, 0.418047827863),
+                           (0.0765906233393, 0.527373281342, 0.6509863541),
+                           (0.146694972056, 0.596725793348, 0.894860986685),
+                           (0.513808585741, 0.069576205858, 0.530890338876),
+                           (0.512343805118, 0.663537132612, 0.037689295973),
+                           (0.47282965018, 0.462176697655, 0.14061843691),
+                           (0.240584597123, 0.778660020591, 0.722913476339),
+                           (0.951271745935, 0.967000673944, 0.890661319684)])
+
+        hull = qhull.ConvexHull(points)
+        assert_allclose(hull.volume, 0.14562013, rtol=1e-07,
+                        err_msg="Volume of random polyhedron is incorrect")
+        assert_allclose(hull.area, 1.6670425, rtol=1e-07,
+                        err_msg="Area of random polyhedron is incorrect")
+
     def _check_barycentric_transforms(self, tri, err_msg="",
                                       unit_cube=False,
                                       unit_cube_tol=0):
@@ -279,8 +310,6 @@ class TestUtilities(object):
             ok = (j != -1) | at_boundary
             assert_(ok.all(), "%s %s" % (err_msg, np.where(~ok)))
 
-    @dec.skipif(NumpyVersion(np.__version__) < '1.6.0',
-                "No einsum in numpy 1.5.x")
     def test_degenerate_barycentric_transforms(self):
         # The triangulation should not produce invalid barycentric
         # transforms that stump the simplex finding
@@ -299,8 +328,6 @@ class TestUtilities(object):
         self._check_barycentric_transforms(tri)
 
     @dec.slow
-    @dec.skipif(NumpyVersion(np.__version__) < '1.6.0',
-                "No einsum in numpy 1.5.x")
     def test_more_barycentric_transforms(self):
         # Triangulate some "nasty" grids
 
@@ -308,6 +335,7 @@ class TestUtilities(object):
 
         npoints = {2: 70, 3: 11, 4: 5, 5: 3}
 
+        _is_32bit_platform = np.intp(0).itemsize < 8
         for ndim in xrange(2, 6):
             # Generate an uniform grid in n-d unit cube
             x = np.linspace(0, 1, npoints[ndim])
@@ -351,10 +379,13 @@ class TestUtilities(object):
             m = (np.random.rand(grid.shape[0]) < 0.2)
             grid[m,:] += 1e6*eps*(np.random.rand(*grid[m,:].shape) - 0.5)
 
-            tri = qhull.Delaunay(grid)
-            self._check_barycentric_transforms(tri, err_msg=err_msg,
-                                               unit_cube=True,
-                                               unit_cube_tol=1e7*eps)
+            if not _is_32bit_platform:
+                # test numerically unstable, and reported to fail on 32-bit
+                # installs
+                tri = qhull.Delaunay(grid)
+                self._check_barycentric_transforms(tri, err_msg=err_msg,
+                                                   unit_cube=True,
+                                                   unit_cube_tol=1e7*eps)
 
 
 class TestVertexNeighborVertices(object):
@@ -400,6 +431,10 @@ class TestDelaunay(object):
         masked_array = np.ma.masked_all(1)
         assert_raises(ValueError, qhull.Delaunay, masked_array)
 
+    def test_array_with_nans_fails(self):
+        points_with_nan = np.array([(0,0), (0,1), (1,1), (1,np.nan)], dtype=np.double)
+        assert_raises(ValueError, qhull.Delaunay, points_with_nan)
+
     def test_nd_simplex(self):
         # simple smoke test: triangulate a n-dimensional simplex
         for nd in xrange(2, 8):
@@ -412,8 +447,8 @@ class TestDelaunay(object):
 
             tri.vertices.sort()
 
-            assert_equal(tri.vertices, np.arange(nd+1, dtype=np.int)[None,:])
-            assert_equal(tri.neighbors, -1 + np.zeros((nd+1), dtype=np.int)[None,:])
+            assert_equal(tri.vertices, np.arange(nd+1, dtype=int)[None,:])
+            assert_equal(tri.neighbors, -1 + np.zeros((nd+1), dtype=int)[None,:])
 
     def test_2d_square(self):
         # simple smoke test: 2d square
@@ -578,6 +613,10 @@ class TestConvexHull:
         masked_array = np.ma.masked_all(1)
         assert_raises(ValueError, qhull.ConvexHull, masked_array)
 
+    def test_array_with_nans_fails(self):
+        points_with_nan = np.array([(0,0), (1,1), (2,np.nan)], dtype=np.double)
+        assert_raises(ValueError, qhull.ConvexHull, points_with_nan)
+
     def test_hull_consistency_tri(self):
         # Check that a convex hull returned by qhull in ndim
         # and the hull constructed from ndim delaunay agree
@@ -635,6 +674,15 @@ class TestConvexHull:
         angle = np.arctan2(y - y.mean(), x - x.mean())
         assert_(np.all(np.diff(np.unwrap(angle)) > 0))
 
+    def test_volume_area(self):
+        # Basic check that we get back the correct volume and area for a cube
+        points = np.array([(0, 0, 0), (0, 1, 0), (1, 0, 0), (1, 1, 0),
+                           (0, 0, 1), (0, 1, 1), (1, 0, 1), (1, 1, 1)])
+        tri = qhull.ConvexHull(points)
+
+        assert_allclose(tri.volume, 1., rtol=1e-14)
+        assert_allclose(tri.area, 6., rtol=1e-14)
+
 
 class TestVoronoi:
     def test_masked_array_fails(self):
@@ -789,6 +837,8 @@ class TestVoronoi:
                                 restart=True)
 
             # -- Check that the incremental mode agrees with upfront mode
+            assert_equal(len(obj.point_region), len(obj2.point_region))
+            assert_equal(len(obj.point_region), len(obj3.point_region))
 
             # The vertices may be in different order or duplicated in
             # the incremental map
@@ -834,5 +884,6 @@ class TestVoronoi:
 
             yield check, name
 
+
 if __name__ == "__main__":
     run_module_suite()
