@@ -9,9 +9,10 @@
 #define _CDF_WRAPPERS_H
 #ifndef _AMOS_WRAPPERS_H
 #include "Python.h"
-#include "cephes/mconf.h"
 #endif
 
+#include "sf_error.h"
+
 #include <numpy/npy_math.h>
 
 extern double cdfbet3_wrap(double p, double x, double b);
