@@ -1,6 +1,6 @@
-/*							ellpj.c
+/*                                                     ellpj.c
  *
- *	Jacobian Elliptic Functions
+ *     Jacobian Elliptic Functions
  *
  *
  *
@@ -40,7 +40,6 @@
  *
  *            Absolute error (* = relative error):
  * arithmetic   function   # trials      peak         rms
- *    DEC       sn           1800       4.5e-16     8.7e-17
  *    IEEE      phi         10000       9.2e-16*    1.4e-16*
  *    IEEE      sn          50000       4.1e-15     4.6e-16
  *    IEEE      cn          40000       3.6e-15     4.4e-16
@@ -53,105 +52,99 @@
  *
  */
 
-/*							ellpj.c		*/
+/*                                                     ellpj.c         */
 
 
 /*
-Cephes Math Library Release 2.0:  April, 1987
-Copyright 1984, 1987 by Stephen L. Moshier
-Direct inquiries to 30 Frost Street, Cambridge, MA 02140
-*/
+ * Cephes Math Library Release 2.0:  April, 1987
+ * Copyright 1984, 1987 by Stephen L. Moshier
+ * Direct inquiries to 30 Frost Street, Cambridge, MA 02140
+ */
 
 #include "mconf.h"
-extern double PIO2, MACHEP;
+extern double MACHEP;
 
-int ellpj( u, m, sn, cn, dn, ph )
+int ellpj(u, m, sn, cn, dn, ph)
 double u, m;
 double *sn, *cn, *dn, *ph;
 {
-double ai, b, phi, t, twon;
-double a[9], c[9];
-int i;
+    double ai, b, phi, t, twon;
+    double a[9], c[9];
+    int i;
 
 
-/* Check for special cases */
+    /* Check for special cases */
 
-if( m < 0.0 || m > 1.0  || npy_isnan(m))
-	{
-	mtherr( "ellpj", DOMAIN );
+    if (m < 0.0 || m > 1.0 || cephes_isnan(m)) {
+	mtherr("ellpj", DOMAIN);
 	*sn = NPY_NAN;
 	*cn = NPY_NAN;
 	*ph = NPY_NAN;
 	*dn = NPY_NAN;
-	return(-1);
-	}
-if( m < 1.0e-9 )
-	{
+	return (-1);
+    }
+    if (m < 1.0e-9) {
 	t = sin(u);
 	b = cos(u);
-	ai = 0.25 * m * (u - t*b);
-	*sn = t - ai*b;
-	*cn = b + ai*t;
+	ai = 0.25 * m * (u - t * b);
+	*sn = t - ai * b;
+	*cn = b + ai * t;
 	*ph = u - ai;
-	*dn = 1.0 - 0.5*m*t*t;
-	return(0);
-	}
+	*dn = 1.0 - 0.5 * m * t * t;
+	return (0);
+    }
 
-if( m >= 0.9999999999 )
-	{
-	ai = 0.25 * (1.0-m);
+    if (m >= 0.9999999999) {
+	ai = 0.25 * (1.0 - m);
 	b = cosh(u);
 	t = tanh(u);
-	phi = 1.0/b;
+	phi = 1.0 / b;
 	twon = b * sinh(u);
-	*sn = t + ai * (twon - u)/(b*b);
-	*ph = 2.0*atan(exp(u)) - PIO2 + ai*(twon - u)/b;
+	*sn = t + ai * (twon - u) / (b * b);
+	*ph = 2.0 * atan(exp(u)) - NPY_PI_2 + ai * (twon - u) / b;
 	ai *= t * phi;
 	*cn = phi - ai * (twon - u);
 	*dn = phi + ai * (twon + u);
-	return(0);
+	return (0);
+    }
+
+
+    /*     A. G. M. scale          */
+    a[0] = 1.0;
+    b = sqrt(1.0 - m);
+    c[0] = sqrt(m);
+    twon = 1.0;
+    i = 0;
+
+    while (fabs(c[i] / a[i]) > MACHEP) {
+	if (i > 7) {
+	    mtherr("ellpj", OVERFLOW);
+	    goto done;
 	}
-
-
-/*	A. G. M. scale		*/
-a[0] = 1.0;
-b = sqrt(1.0 - m);
-c[0] = sqrt(m);
-twon = 1.0;
-i = 0;
-
-while( fabs(c[i]/a[i]) > MACHEP )
-	{
-	if( i > 7 )
-		{
-		mtherr( "ellpj", OVERFLOW );
-		goto done;
-		}
 	ai = a[i];
 	++i;
-	c[i] = ( ai - b )/2.0;
-	t = sqrt( ai * b );
-	a[i] = ( ai + b )/2.0;
+	c[i] = (ai - b) / 2.0;
+	t = sqrt(ai * b);
+	a[i] = (ai + b) / 2.0;
 	b = t;
 	twon *= 2.0;
-	}
+    }
 
-done:
+  done:
 
-/* backward recurrence */
-phi = twon * a[i] * u;
-do
-	{
+    /* backward recurrence */
+    phi = twon * a[i] * u;
+    do {
 	t = c[i] * sin(phi) / a[i];
 	b = phi;
-	phi = (asin(t) + phi)/2.0;
-	}
-while( --i );
+	phi = (asin(t) + phi) / 2.0;
+    }
+    while (--i);
 
-*sn = sin(phi);
-t = cos(phi);
-*cn = t;
-*dn = t/cos(phi-b);
-*ph = phi;
-return(0);
+    *sn = sin(phi);
+    t = cos(phi);
+    *cn = t;
+    *dn = t / cos(phi - b);
+    *ph = phi;
+    return (0);
 }
