@@ -3,7 +3,8 @@
  */
 
 #include "specfun_wrappers.h"
-#include <stdio.h>
+
+#include "sf_error.h"
 
 #define CADDR(z) (double *)(&((z).real)), (double*)(&((z).imag))
 #define F2C_CST(z) (double *)&((z)->real), (double *)&((z)->imag)
@@ -27,15 +28,15 @@ extern double cephes_struve(double, double);
 
 extern void F_FUNC(cgama,CGAMA)(double*,double*,int*,double*,double*);
 extern void F_FUNC(cpsi,CPSI)(double*,double*,double*,double*);
-extern void F_FUNC(hygfz,HYGFZ)(double*,double*,double*,Py_complex*,Py_complex*);
-extern void F_FUNC(cchg,CCHG)(double*,double*,Py_complex*,Py_complex*);
+extern void F_FUNC(hygfz,HYGFZ)(double*,double*,double*,npy_cdouble*,npy_cdouble*);
+extern void F_FUNC(cchg,CCHG)(double*,double*,npy_cdouble*,npy_cdouble*);
 extern void F_FUNC(chgm,CHGM)(double*,double*,double*,double*);
 extern void F_FUNC(chgu,CHGU)(double*,double*,double*,double*,int*);
 extern void F_FUNC(itairy,ITAIRY)(double*,double*,double*,double*,double*);
 extern void F_FUNC(e1xb,E1XB)(double*,double*);
-extern void F_FUNC(e1z,E1Z)(Py_complex*,Py_complex*);
+extern void F_FUNC(e1z,E1Z)(npy_cdouble*,npy_cdouble*);
 extern void F_FUNC(eix,EIX)(double*,double*);
-extern void F_FUNC(cerror,CERROR)(Py_complex*,Py_complex*);
+extern void F_FUNC(cerror,CERROR)(npy_cdouble*,npy_cdouble*);
 extern void F_FUNC(stvh0,STVH0)(double*,double*);
 extern void F_FUNC(stvh1,STVH1)(double*,double*);
 extern void F_FUNC(stvhv,STVHV)(double*,double*,double*);
@@ -50,8 +51,8 @@ extern void F_FUNC(itjya,ITJYA)(double*,double*,double*);
 extern void F_FUNC(ittjya,ITTJYA)(double*,double*,double*);
 extern void F_FUNC(itika,ITIKA)(double*,double*,double*);
 extern void F_FUNC(ittika,ITTIKA)(double*,double*,double*);
-extern void F_FUNC(cfc,CFC)(Py_complex*,Py_complex*,Py_complex*);
-extern void F_FUNC(cfs,CFS)(Py_complex*,Py_complex*,Py_complex*);
+extern void F_FUNC(cfc,CFC)(npy_cdouble*,npy_cdouble*,npy_cdouble*);
+extern void F_FUNC(cfs,CFS)(npy_cdouble*,npy_cdouble*,npy_cdouble*);
 extern void F_FUNC(cva2,CVA2)(int*,int*,double*,double*);
 extern void F_FUNC(mtu0,MTU0)(int*,int*,double*,double*,double*,double*);
 extern void F_FUNC(mtu12,MTU12)(int*,int*,int*,double*,double*,double*,double*,double*,double*);
@@ -69,24 +70,24 @@ extern void F_FUNC(ffk,FFK)(int*,double*,double*,double*,double*,double*,double*
 /* This must be linked with fortran
  */
 
-Py_complex cgamma_wrap( Py_complex z) {
+npy_cdouble cgamma_wrap( npy_cdouble z) {
   int kf = 1;
-  Py_complex cy;
+  npy_cdouble cy;
 
   F_FUNC(cgama,CGAMA)(CADDR(z), &kf, CADDR(cy));
   return cy;
 }
 
-Py_complex clngamma_wrap( Py_complex z) {
+npy_cdouble clngamma_wrap( npy_cdouble z) {
   int kf = 0;
-  Py_complex cy;
+  npy_cdouble cy;
 
   F_FUNC(cgama,CGAMA)(CADDR(z), &kf, CADDR(cy));
   return cy;
 }
 
-Py_complex cpsi_wrap( Py_complex z) {
-  Py_complex cy;
+npy_cdouble cpsi_wrap( npy_cdouble z) {
+  npy_cdouble cy;
   
   if (IMAG(z)==0.0) {
     REAL(cy) = cephes_psi(REAL(z));
@@ -98,10 +99,10 @@ Py_complex cpsi_wrap( Py_complex z) {
   return cy;
 }
 
-Py_complex crgamma_wrap( Py_complex z) {
+npy_cdouble crgamma_wrap( npy_cdouble z) {
   int kf = 1;
-  Py_complex cy;
-  Py_complex cy2;
+  npy_cdouble cy;
+  npy_cdouble cy2;
   double magsq;
 
   F_FUNC(cgama,CGAMA)(CADDR(z), &kf, CADDR(cy));
@@ -111,14 +112,15 @@ Py_complex crgamma_wrap( Py_complex z) {
   return cy2;
 }
 
-Py_complex chyp2f1_wrap( double a, double b, double c, Py_complex z) {
-  Py_complex outz;
+npy_cdouble chyp2f1_wrap( double a, double b, double c, npy_cdouble z) {
+  npy_cdouble outz;
   int l1, l0;
  
  
   l0 = ((c == floor(c)) && (c < 0));
   l1 = ((fabs(1-REAL(z)) < 1e-15) && (IMAG(z) == 0) && (c-a-b <= 0));
   if (l0 || l1) {
+    sf_error("chyp2f1", SF_ERROR_OVERFLOW, NULL);
     REAL(outz) = NPY_INFINITY;
     IMAG(outz) = 0.0;
     return outz;
@@ -127,11 +129,12 @@ Py_complex chyp2f1_wrap( double a, double b, double c, Py_complex z) {
   return outz;
 }
 
-Py_complex chyp1f1_wrap(double a, double b, Py_complex z) {
-  Py_complex outz;
+npy_cdouble chyp1f1_wrap(double a, double b, npy_cdouble z) {
+  npy_cdouble outz;
 
   F_FUNC(cchg,CCHG)(&a, &b, &z, &outz);
   if (REAL(outz) == 1e300) {
+    sf_error("chyp1f1", SF_ERROR_OVERFLOW, NULL);
     REAL(outz) = NPY_INFINITY;
   }
   return outz;
@@ -143,7 +146,10 @@ double hypU_wrap(double a, double b, double x) {
   int md; /* method code --- not returned */
 
   F_FUNC(chgu,CHGU)(&a, &b, &x, &out, &md);
-  if (out == 1e300) out = NPY_INFINITY;
+  if (out == 1e300) {
+      sf_error("hypU", SF_ERROR_OVERFLOW, NULL);
+      out = NPY_INFINITY;
+  }
   return out;
   
 }
@@ -153,6 +159,7 @@ double hyp1f1_wrap(double a, double b, double x) {
  
    F_FUNC(chgm,CHGM)(&a, &b, &x, &outy);
    if (outy == 1e300) {
+     sf_error("hyp1f1", SF_ERROR_OVERFLOW, NULL);
      outy = NPY_INFINITY;
    }
    return outy;
@@ -183,15 +190,15 @@ double exp1_wrap(double x) {
   double out;
   
   F_FUNC(e1xb,E1XB)(&x, &out);
-  CONVINF(out);
+  CONVINF("exp1", out);
   return out;
 }
 
-Py_complex cexp1_wrap(Py_complex z) {
-  Py_complex outz;
+npy_cdouble cexp1_wrap(npy_cdouble z) {
+  npy_cdouble outz;
   
   F_FUNC(e1z,E1Z)(&z, &outz);
-  ZCONVINF(outz);
+  ZCONVINF("cexp1", outz);
   return outz;
 }
 
@@ -199,20 +206,20 @@ double expi_wrap(double x) {
   double out;
   
   F_FUNC(eix,EIX)(&x, &out);
-  CONVINF(out);
+  CONVINF("expi", out);
   return out;
 }
 
-Py_complex cexpi_wrap(Py_complex z) {
-  Py_complex outz;
+npy_cdouble cexpi_wrap(npy_cdouble z) {
+  npy_cdouble outz;
 
   F_FUNC(eixz,EIXZ)(&z, &outz);
-  ZCONVINF(outz);
+  ZCONVINF("cexpi", outz);
   return outz;
 }
 
-Py_complex cerf_wrap(Py_complex z) {
-  Py_complex outz;
+npy_cdouble cerf_wrap(npy_cdouble z) {
+  npy_cdouble outz;
   
   F_FUNC(cerror,CERROR)(&z, &outz);
   return outz;
@@ -242,15 +249,15 @@ double struve_wrap(double v, double x) {
   }
   else if (v==0.0) {
     F_FUNC(stvh0,STVH0)(&x,&out);
-    CONVINF(out);
+    CONVINF("struve", out);
   }
   else if (v==1.0) {
     F_FUNC(stvh1,STVH1)(&x,&out);
-    CONVINF(out);
+    CONVINF("struve", out);
   }
   else {
     F_FUNC(stvhv,STVHV)(&v,&x,&out);
-    CONVINF(out);
+    CONVINF("struve", out);
   }
   if (flag) out = -out;
   return out;
@@ -264,14 +271,14 @@ double modstruve_wrap(double v, double x) {
   if (v==0.0) {
     if (x < 0) {x = -x; flag=1;}
     F_FUNC(stvl0,STVL0)(&x,&out);
-    CONVINF(out);
+    CONVINF("modstruve", out);
     if (flag) out = -out;
     return out;
   }
   if (v==1.0) {
     if (x < 0) x=-x;
     F_FUNC(stvl1,STVL1)(&x,&out);
-    CONVINF(out);
+    CONVINF("modstruve", out);
     return out;
   }
   if (x<0) {
@@ -279,7 +286,7 @@ double modstruve_wrap(double v, double x) {
     flag = 1;
   }
   F_FUNC(stvlv,STVLV)(&v,&x,&out);
-  CONVINF(out);
+  CONVINF("modstruve", out);
   if (flag && (!((int)floor(v) % 2))) out = -out;
   return out;  
 }
@@ -289,7 +296,7 @@ double itstruve0_wrap(double x) {
 
   if (x<0) x=-x;
   F_FUNC(itsh0,ITSH0)(&x,&out);
-  CONVINF(out);
+  CONVINF("itstruve0", out);
   return out;
 }
 
@@ -299,9 +306,9 @@ double it2struve0_wrap(double x) {
   
   if (x<0) {x=-x; flag=1;}
   F_FUNC(itth0,ITTH0)(&x,&out);
-  CONVINF(out);
+  CONVINF("it2struve0", out);
   if (flag) {
-    out = PI - out;
+    out = NPY_PI - out;
   }
   return out;
 }
@@ -311,105 +318,105 @@ double itmodstruve0_wrap(double x) {
 
   if (x<0) x=-x;
   F_FUNC(itsl0,ITSL0)(&x,&out);
-  CONVINF(out);
+  CONVINF("itmodstruve0", out);
   return out;
 }
 
 
 double ber_wrap(double x)
 {
-  Py_complex Be, Ke, Bep, Kep;
+  npy_cdouble Be, Ke, Bep, Kep;
 
   if (x<0) x=-x;
   F_FUNC(klvna,KLVNA)(&x, CADDR(Be), CADDR(Ke), CADDR(Bep), CADDR(Kep));
-  ZCONVINF(Be);
+  ZCONVINF("ber", Be);
   return REAL(Be);
 }
 
 double bei_wrap(double x)
 {
-  Py_complex Be, Ke, Bep, Kep;
+  npy_cdouble Be, Ke, Bep, Kep;
 
   if (x<0) x=-x;
   F_FUNC(klvna,KLVNA)(&x, CADDR(Be), CADDR(Ke), CADDR(Bep), CADDR(Kep));
-  ZCONVINF(Be);
+  ZCONVINF("bei", Be);
   return IMAG(Be);
 }
 
 double ker_wrap(double x)
 {
-  Py_complex Be, Ke, Bep, Kep;
+  npy_cdouble Be, Ke, Bep, Kep;
 
   if (x<0) return NPY_NAN;
   F_FUNC(klvna,KLVNA)(&x, CADDR(Be), CADDR(Ke), CADDR(Bep), CADDR(Kep));
-  ZCONVINF(Ke);
+  ZCONVINF("ker", Ke);
   return REAL(Ke);  
 }
 
 double kei_wrap(double x)
 {
-  Py_complex Be, Ke, Bep, Kep;
+  npy_cdouble Be, Ke, Bep, Kep;
 
   if (x<0) return NPY_NAN;
   F_FUNC(klvna,KLVNA)(&x, CADDR(Be), CADDR(Ke), CADDR(Bep), CADDR(Kep));
-  ZCONVINF(Ke);
+  ZCONVINF("kei", Ke);
   return IMAG(Ke);  
 }
 
 double berp_wrap(double x)
 {
-  Py_complex Be, Ke, Bep, Kep;
+  npy_cdouble Be, Ke, Bep, Kep;
   int flag = 0;
 
   if (x<0) {x=-x; flag=1;}
   F_FUNC(klvna,KLVNA)(&x, CADDR(Be), CADDR(Ke), CADDR(Bep), CADDR(Kep));
-  ZCONVINF(Bep);
+  ZCONVINF("berp", Bep);
   if (flag) return -REAL(Bep);
   return REAL(Bep);
 }
 
 double beip_wrap(double x)
 {
-  Py_complex Be, Ke, Bep, Kep;
+  npy_cdouble Be, Ke, Bep, Kep;
   int flag = 0;
 
   if (x<0) {x=-x; flag=1;}
   F_FUNC(klvna,KLVNA)(&x, CADDR(Be), CADDR(Ke), CADDR(Bep), CADDR(Kep));
-  ZCONVINF(Bep);
+  ZCONVINF("beip", Bep);
   if (flag) return -IMAG(Bep);
   return IMAG(Bep);
 }
 
 double kerp_wrap(double x)
 {
-  Py_complex Be, Ke, Bep, Kep;
+  npy_cdouble Be, Ke, Bep, Kep;
 
   if (x<0) return NPY_NAN;
   F_FUNC(klvna,KLVNA)(&x, CADDR(Be), CADDR(Ke), CADDR(Bep), CADDR(Kep));
-  ZCONVINF(Kep);
+  ZCONVINF("kerp", Kep);
   return REAL(Kep);  
 }
 
 double keip_wrap(double x)
 {
-  Py_complex Be, Ke, Bep, Kep;
+  npy_cdouble Be, Ke, Bep, Kep;
 
   if (x<0) return NPY_NAN;
   F_FUNC(klvna,KLVNA)(&x, CADDR(Be), CADDR(Ke), CADDR(Bep), CADDR(Kep));
-  ZCONVINF(Kep);
+  ZCONVINF("keip", Kep);
   return IMAG(Kep);  
 }
 
 
-int kelvin_wrap(double x, Py_complex *Be, Py_complex *Ke, Py_complex *Bep, Py_complex *Kep) {
+int kelvin_wrap(double x, npy_cdouble *Be, npy_cdouble *Ke, npy_cdouble *Bep, npy_cdouble *Kep) {
   int flag = 0;
   
   if (x<0) {x=-x; flag=1;}
   F_FUNC(klvna,KLVNA)(&x, F2C_CST(Be), F2C_CST(Ke), F2C_CST(Bep), F2C_CST(Kep));
-  ZCONVINF(*Be);
-  ZCONVINF(*Ke);
-  ZCONVINF(*Bep);
-  ZCONVINF(*Kep);
+  ZCONVINF("klvna", *Be);
+  ZCONVINF("klvna", *Ke);
+  ZCONVINF("klvna", *Bep);
+  ZCONVINF("klvna", *Kep);
   if (flag) {
     REAL(*Bep) = -REAL(*Bep);
     IMAG(*Bep) = -IMAG(*Bep);
@@ -484,9 +491,9 @@ int it2i0k0_wrap(double x, double *i0int, double *k0int)
 
 /* Fresnel integrals of complex numbers */
 
-int cfresnl_wrap(Py_complex z, Py_complex *zfs, Py_complex *zfc)
+int cfresnl_wrap(npy_cdouble z, npy_cdouble *zfs, npy_cdouble *zfc)
 {
-  Py_complex zfd;
+  npy_cdouble zfd;
   F_FUNC(cfs,CFS)(&z,zfs,&zfd);
   F_FUNC(cfc,CFC)(&z,zfc,&zfd); 
   return 0;
@@ -498,47 +505,115 @@ double cem_cva_wrap(double m, double q) {
   int int_m, kd=1;
   double out;
 
-  if ((m < 0) || (m != floor(m))) 
+  if ((m < 0) || (m != floor(m))) {
+    sf_error("cem_cva", SF_ERROR_DOMAIN, NULL);
     return NPY_NAN;
+  }
   int_m = (int )m;
+  if (q < 0) {
+    /* http://dlmf.nist.gov/28.2#E26 */
+    if (int_m % 2 == 0) {
+      return cem_cva_wrap(m, -q);
+    }
+    else {
+      return sem_cva_wrap(m, -q);
+    }
+  }
+
   if (int_m % 2) kd=2;
   F_FUNC(cva2,CVA2)(&kd, &int_m, &q, &out);
-  return out;               
+  return out;
 }
 
 double sem_cva_wrap(double m, double q) {
   int int_m, kd=4;
   double out;
 
-  if ((m < 1) || (m != floor(m))) 
+  if ((m <= 0) || (m != floor(m))) {
+    sf_error("cem_cva", SF_ERROR_DOMAIN, NULL);
     return NPY_NAN;
-  int_m = (int )m;
+  }
+  int_m = (int)m;
+  if (q < 0) {
+    /* http://dlmf.nist.gov/28.2#E26 */
+    if (int_m % 2 == 0) {
+      return sem_cva_wrap(m, -q);
+    }
+    else {
+      return cem_cva_wrap(m, -q);
+    }
+  }
   if (int_m % 2) kd=3;
   F_FUNC(cva2,CVA2)(&kd, &int_m, &q, &out);
-  return out;               
+  return out;
 }
 
 /* Mathieu functions */
 int cem_wrap(double m, double q, double x, double *csf, double *csd)
 {
-  int int_m, kf=1;
-  if ((m < 1) || (m != floor(m)) || (q<0)) {
+  int int_m, kf=1, sgn;
+  double f, d;
+  if ((m < 0) || (m != floor(m))) {
     *csf = NPY_NAN;
     *csd = NPY_NAN;
+    sf_error("cem", SF_ERROR_DOMAIN, NULL);
+    return -1;
+  }
+  int_m = (int)m;
+  if (q < 0) {
+      /* http://dlmf.nist.gov/28.2#E34 */
+      if (int_m % 2 == 0) {
+        sgn = ((int_m/2) % 2 == 0) ? 1 : -1;
+        cem_wrap(m, -q, 90 - x, &f, &d);
+        *csf =  sgn * f;
+        *csd = -sgn * d;
+        return 0;
+      }
+      else {
+        sgn = ((int_m/2) % 2 == 0) ? 1 : -1;
+        sem_wrap(m, -q, 90 - x, &f, &d);
+        *csf =  sgn * f;
+        *csd = -sgn * d;
+        return 0;
+      }
   }
-  int_m = (int )m;
   F_FUNC(mtu0,MTU0)(&kf,&int_m, &q, &x, csf, csd);
   return 0;  
 }
 
 int sem_wrap(double m, double q, double x, double *csf, double *csd)
 {
-  int int_m, kf=2;
-  if ((m < 1) || (m != floor(m)) || (q<0)) {
+  int int_m, kf=2, sgn;
+  double f, d;
+  if ((m < 0) || (m != floor(m))) {
     *csf = NPY_NAN;
     *csd = NPY_NAN;
+    sf_error("sem", SF_ERROR_DOMAIN, NULL);
+    return -1;
+  }
+  int_m = (int)m;
+  if (int_m == 0) {
+    *csf = 0;
+    *csd = 0;
+    return 0;
+  }
+  if (q < 0) {
+      /* http://dlmf.nist.gov/28.2#E34 */
+      if (int_m % 2 == 0) {
+        sgn = ((int_m/2) % 2 == 0) ? -1 : 1;
+        sem_wrap(m, -q, 90 - x, &f, &d);
+        *csf =  sgn * f;
+        *csd = -sgn * d;
+        return 0;
+      }
+      else {
+        sgn = ((int_m/2) % 2 == 0) ? 1 : -1;
+        cem_wrap(m, -q, 90 - x, &f, &d);
+        *csf =  sgn * f;
+        *csd = -sgn * d;
+        return 0;
+      }
   }
-  int_m = (int )m;
   F_FUNC(mtu0,MTU0)(&kf,&int_m, &q, &x, csf, csd);
   return 0;  
 }
@@ -549,9 +624,11 @@ int mcm1_wrap(double m, double q, double x, double *f1r, double *d1r)
   int int_m, kf=1, kc=1;
   double f2r, d2r;
 
-  if ((m < 1) || (m != floor(m)) || (q<0)) {
+  if ((m < 0) || (m != floor(m)) || (q<0)) {
     *f1r = NPY_NAN;
     *d1r = NPY_NAN;
+    sf_error("mcm1", SF_ERROR_DOMAIN, NULL);
+    return -1;
   }
   int_m = (int )m;
   F_FUNC(mtu12,MTU12)(&kf,&kc,&int_m, &q, &x, f1r, d1r, &f2r, &d2r);
@@ -566,6 +643,8 @@ int msm1_wrap(double m, double q, double x, double *f1r, double *d1r)
   if ((m < 1) || (m != floor(m)) || (q<0)) {
     *f1r = NPY_NAN;
     *d1r = NPY_NAN;
+    sf_error("msm1", SF_ERROR_DOMAIN, NULL);
+    return -1;
   }
   int_m = (int )m;
   F_FUNC(mtu12,MTU12)(&kf,&kc,&int_m, &q, &x, f1r, d1r, &f2r, &d2r);
@@ -577,9 +656,11 @@ int mcm2_wrap(double m, double q, double x, double *f2r, double *d2r)
   int int_m, kf=1, kc=2;
   double f1r, d1r;
 
-  if ((m < 1) || (m != floor(m)) || (q<0)) {
+  if ((m < 0) || (m != floor(m)) || (q<0)) {
     *f2r = NPY_NAN;
     *d2r = NPY_NAN;
+    sf_error("mcm2", SF_ERROR_DOMAIN, NULL);
+    return -1;
   }
   int_m = (int )m;
   F_FUNC(mtu12,MTU12)(&kf,&kc,&int_m, &q, &x, &f1r, &d1r, f2r, d2r);
@@ -594,6 +675,8 @@ int msm2_wrap(double m, double q, double x, double *f2r, double *d2r)
   if ((m < 1) || (m != floor(m)) || (q<0)) {
     *f2r = NPY_NAN;
     *d2r = NPY_NAN;
+    sf_error("msm2", SF_ERROR_DOMAIN, NULL);
+    return -1;
   }
   int_m = (int )m;
   F_FUNC(mtu12,MTU12)(&kf,&kc,&int_m, &q, &x, &f1r, &d1r, f2r, d2r);
@@ -608,7 +691,7 @@ double pmv_wrap(double m, double v, double x){
   if (m != floor(m)) return NPY_NAN;
   int_m = (int ) m;
   F_FUNC(lpmv,LPMV)(&v, &int_m, &x, &out);
-  CONVINF(out);
+  CONVINF("pmv", out);
   return out;
 }
 
@@ -643,7 +726,7 @@ int pbdv_wrap(double v, double x, double *pdf, double *pdd) {
   num = ABS((int)v) + 2;
   dv = (double *)PyMem_Malloc(sizeof(double)*2*num);
   if (dv==NULL) {
-    printf("Warning: Memory allocation error.\n"); 
+    sf_error("pbdv", SF_ERROR_OTHER, "memory allocation error");
     *pdf = NPY_NAN;
     *pdd = NPY_NAN;
     return -1;
@@ -663,7 +746,7 @@ int pbvv_wrap(double v, double x, double *pvf, double *pvd) {
   num = ABS((int)v) + 2;
   vv = (double *)PyMem_Malloc(sizeof(double)*2*num);
   if (vv==NULL) {
-    printf("Warning: Memory allocation error.\n"); 
+    sf_error("pbvv", SF_ERROR_OTHER, "memory allocation error");
     *pvf = NPY_NAN;
     *pvd = NPY_NAN;
     return -1;
@@ -687,7 +770,7 @@ double prolate_segv_wrap(double m, double n, double c)
   int_n = (int) n;
   eg = (double *)PyMem_Malloc(sizeof(double)*(n-m+2));
   if (eg==NULL) {
-    printf("Warning: Memory allocation error.\n"); 
+    sf_error("prolate_segv", SF_ERROR_OTHER, "memory allocation error");
     return NPY_NAN;
   }
   F_FUNC(segv,SEGV)(&int_m,&int_n,&c,&kd,&cv,eg);
@@ -708,7 +791,7 @@ double oblate_segv_wrap(double m, double n, double c)
   int_n = (int) n;
   eg = (double *)PyMem_Malloc(sizeof(double)*(n-m+2));
   if (eg==NULL) {
-    printf("Warning: Memory allocation error.\n"); 
+    sf_error("oblate_segv", SF_ERROR_OTHER, "memory allocation error");
     return NPY_NAN;
   }
   F_FUNC(segv,SEGV)(&int_m,&int_n,&c,&kd,&cv,eg);
@@ -725,6 +808,7 @@ double prolate_aswfa_nocv_wrap(double m, double n, double c, double x, double *s
 
   if ((x >=1) || (x <=-1) || (m<0) || (n<m) || \
       (m!=floor(m)) || (n!=floor(n)) || ((n-m)>198)) {
+    sf_error("prolate_aswfa_nocv", SF_ERROR_DOMAIN, NULL);
     *s1d = NPY_NAN;
     return NPY_NAN;
   }
@@ -732,7 +816,7 @@ double prolate_aswfa_nocv_wrap(double m, double n, double c, double x, double *s
   int_n = (int )n;
   eg = (double *)PyMem_Malloc(sizeof(double)*(n-m+2));
   if (eg==NULL) {
-    printf("Warning: Memory allocation error.\n"); 
+    sf_error("prolate_aswfa_nocv", SF_ERROR_OTHER, "memory allocation error");
     *s1d = NPY_NAN;
     return NPY_NAN;
   }
@@ -751,6 +835,7 @@ double oblate_aswfa_nocv_wrap(double m, double n, double c, double x, double *s1
 
   if ((x >=1) || (x <=-1) || (m<0) || (n<m) || \
       (m!=floor(m)) || (n!=floor(n)) || ((n-m)>198)) {
+    sf_error("oblate_aswfa_nocv", SF_ERROR_DOMAIN, NULL);
     *s1d = NPY_NAN;
     return NPY_NAN;
   }
@@ -758,7 +843,7 @@ double oblate_aswfa_nocv_wrap(double m, double n, double c, double x, double *s1
   int_n = (int )n;
   eg = (double *)PyMem_Malloc(sizeof(double)*(n-m+2));
   if (eg==NULL) {
-    printf("Warning: Memory allocation error.\n"); 
+    sf_error("oblate_aswfa_nocv", SF_ERROR_OTHER, "memory allocation error");
     *s1d = NPY_NAN;
     return NPY_NAN;
   }
@@ -776,6 +861,7 @@ int prolate_aswfa_wrap(double m, double n, double c, double cv, double x, double
 
   if ((x >=1) || (x <=-1) || (m<0) || (n<m) || \
       (m!=floor(m)) || (n!=floor(n))) {
+    sf_error("prolate_aswfa", SF_ERROR_DOMAIN, NULL);
     *s1f = NPY_NAN;
     *s1d = NPY_NAN;
     return 0;
@@ -794,6 +880,7 @@ int oblate_aswfa_wrap(double m, double n, double c, double cv, double x, double
 
   if ((x >=1) || (x <=-1) || (m<0) || (n<m) || \
       (m!=floor(m)) || (n!=floor(n))) {
+    sf_error("oblate_aswfa", SF_ERROR_DOMAIN, NULL);
     *s1f = NPY_NAN;
     *s1d = NPY_NAN;
     return 0;
@@ -813,6 +900,7 @@ double prolate_radial1_nocv_wrap(double m, double n, double c, double x, double
 
   if ((x <=1.0) || (m<0) || (n<m) || \
      (m!=floor(m)) || (n!=floor(n)) || ((n-m)>198)) {
+    sf_error("prolate_radial1_nocv", SF_ERROR_DOMAIN, NULL);
     *r1d = NPY_NAN;
     return NPY_NAN;
   }
@@ -820,7 +908,7 @@ double prolate_radial1_nocv_wrap(double m, double n, double c, double x, double
   int_n = (int )n;
   eg = (double *)PyMem_Malloc(sizeof(double)*(n-m+2));
   if (eg==NULL) {
-    printf("Warning: Memory allocation error.\n"); 
+    sf_error("prolate_radial1_nocv", SF_ERROR_OTHER, "memory allocation error");
     *r1d = NPY_NAN;
     return NPY_NAN;
   }
@@ -838,6 +926,7 @@ double prolate_radial2_nocv_wrap(double m, double n, double c, double x, double
 
   if ((x <=1.0) || (m<0) || (n<m) || \
      (m!=floor(m)) || (n!=floor(n)) || ((n-m)>198)) {
+    sf_error("prolate_radial2_nocv", SF_ERROR_DOMAIN, NULL);
     *r2d = NPY_NAN;
     return NPY_NAN;
   }
@@ -845,7 +934,7 @@ double prolate_radial2_nocv_wrap(double m, double n, double c, double x, double
   int_n = (int )n;
   eg = (double *)PyMem_Malloc(sizeof(double)*(n-m+2));
   if (eg==NULL) {
-    printf("Warning: Memory allocation error.\n"); 
+    sf_error("prolate_radial2_nocv", SF_ERROR_OTHER, "memory allocation error");
     *r2d = NPY_NAN;
     return NPY_NAN;
   }
@@ -863,6 +952,7 @@ int prolate_radial1_wrap(double m, double n, double c, double cv, double x, doub
 
   if ((x <= 1.0) || (m<0) || (n<m) || \
      (m!=floor(m)) || (n!=floor(n))) {
+    sf_error("prolate_radial1", SF_ERROR_DOMAIN, NULL);
     *r1f = NPY_NAN;
     *r1d = NPY_NAN;
     return 0;
@@ -881,6 +971,7 @@ int prolate_radial2_wrap(double m, double n, double c, double cv, double x, doub
 
   if ((x <= 1.0) || (m<0) || (n<m) || \
      (m!=floor(m)) || (n!=floor(n))) {
+    sf_error("prolate_radial2", SF_ERROR_DOMAIN, NULL);
     *r2f = NPY_NAN;
     *r2d = NPY_NAN;
     return 0;
@@ -899,6 +990,7 @@ double oblate_radial1_nocv_wrap(double m, double n, double c, double x, double *
 
   if ((x < 0.0) || (m<0) || (n<m) || \
      (m!=floor(m)) || (n!=floor(n)) || ((n-m)>198)) {
+    sf_error("oblate_radial1_nocv", SF_ERROR_DOMAIN, NULL);
     *r1d = NPY_NAN;
     return NPY_NAN;
   }
@@ -906,7 +998,7 @@ double oblate_radial1_nocv_wrap(double m, double n, double c, double x, double *
   int_n = (int )n;
   eg = (double *)PyMem_Malloc(sizeof(double)*(n-m+2));
   if (eg==NULL) {
-    printf("Warning: Memory allocation error.\n"); 
+    sf_error("oblate_radial1_nocv", SF_ERROR_OTHER, "memory allocation error");
     *r1d = NPY_NAN;
     return NPY_NAN;
   }
@@ -924,6 +1016,7 @@ double oblate_radial2_nocv_wrap(double m, double n, double c, double x, double *
 
   if ((x < 0.0) || (m<0) || (n<m) || \
      (m!=floor(m)) || (n!=floor(n)) || ((n-m)>198)) {
+    sf_error("oblate_radial2_nocv", SF_ERROR_DOMAIN, NULL);
     *r2d = NPY_NAN;
     return NPY_NAN;
   }
@@ -931,7 +1024,7 @@ double oblate_radial2_nocv_wrap(double m, double n, double c, double x, double *
   int_n = (int )n;
   eg = (double *)PyMem_Malloc(sizeof(double)*(n-m+2));
   if (eg==NULL) {
-    printf("Warning: Memory allocation error.\n"); 
+    sf_error("oblate_radial2_nocv", SF_ERROR_OTHER, "memory allocation error");
     *r2d = NPY_NAN;
     return NPY_NAN;
   }
@@ -949,6 +1042,7 @@ int oblate_radial1_wrap(double m, double n, double c, double cv, double x, doubl
 
   if ((x <0.0) || (m<0) || (n<m) || \
      (m!=floor(m)) || (n!=floor(n))) {
+    sf_error("oblate_radial1", SF_ERROR_DOMAIN, NULL);
     *r1f = NPY_NAN;
     *r1d = NPY_NAN;
     return 0;
@@ -967,6 +1061,7 @@ int oblate_radial2_wrap(double m, double n, double c, double cv, double x, doubl
 
   if ((x <0.0) || (m<0) || (n<m) || \
      (m!=floor(m)) || (n!=floor(n))) {
+    sf_error("oblate_radial2", SF_ERROR_DOMAIN, NULL);
     *r2f = NPY_NAN;
     *r2d = NPY_NAN;
     return 0;
@@ -978,7 +1073,7 @@ int oblate_radial2_wrap(double m, double n, double c, double cv, double x, doubl
 }
 
 
-int modified_fresnel_plus_wrap(double x, Py_complex *Fplus, Py_complex *Kplus)
+int modified_fresnel_plus_wrap(double x, npy_cdouble *Fplus, npy_cdouble *Kplus)
 {
   int ks=0;
   double fm, fa, gm, ga;
@@ -987,7 +1082,7 @@ int modified_fresnel_plus_wrap(double x, Py_complex *Fplus, Py_complex *Kplus)
   return 0;
 }
 
-int modified_fresnel_minus_wrap(double x, Py_complex *Fminus, Py_complex *Kminus)
+int modified_fresnel_minus_wrap(double x, npy_cdouble *Fminus, npy_cdouble *Kminus)
 {
   int ks=1;
   double fm, fa, gm, ga;
