@@ -24,8 +24,8 @@ from __future__ import division, print_function, absolute_import
 
 _os = __import__('os')
 
-from scipy.lib.six import builtins
-from scipy.lib.six import string_types
+from scipy._lib.six import builtins
+from scipy._lib.six import string_types
 
 _open = builtins.open
 
