@@ -251,7 +251,7 @@ class dummy_array(object):
                     end += dim_len
                 # the following is list like behavior,
                 # which isn't adhered to by arrays.
-                # FIX THIS ANOMOLY IN NUMERIC!
+                # FIX THIS ANOMALY IN NUMERIC!
                 if beg < 0:
                     beg = 0
                 if beg > dim_len:
@@ -288,7 +288,7 @@ class dummy_array(object):
             else:
                 if index < 0:
                     index += dim_len
-                if index >= 0 and index < dim_len:
+                if 0 <= index < dim_len:
                     # this reduces the array dimensions by one
                     pass
                 else:
@@ -338,5 +338,5 @@ def reduction(ary,axis=0):
 
 
 def take(ary,axis=0):
-    raise NotImplemented
+    raise NotImplementedError
 # and all the rest
