@@ -1,14 +1,12 @@
 from __future__ import absolute_import, print_function
 
-from numpy.testing import TestCase, assert_equal
+from numpy.testing import TestCase, assert_equal, run_module_suite
 
 from scipy.weave import ast_tools
 
 
 class TestHarvestVariables(TestCase):
-    """ Not much testing going on here, but
-        at least it is a flame test.
-    """
+    """ Not much testing going on here, but at least it is a flame test."""
     def generic_check(self,expr,desired):
         import parser
         ast_list = parser.suite(expr).tolist()
@@ -16,14 +14,10 @@ class TestHarvestVariables(TestCase):
         assert_equal(actual,desired,expr)
 
     def test_simple_expr(self):
-        """convert simple expr to blitz
-
-           a[:1:2] = b[:1+i+2:]
-        """
+        # Convert simple expr to blitz
         expr = "a[:1:2] = b[:1+i+2:]"
         desired = ['a','b','i']
         self.generic_check(expr,desired)
 
 if __name__ == "__main__":
-    import nose
-    nose.run(argv=['', __file__])
+    run_module_suite()
